
	----------------------------------------------------------------------
	-- 	Leatrix Maps 8.3.04 (12th February 2020)
	----------------------------------------------------------------------

	-- 10:Func, 20:Comm, 30:Evnt, 40:Panl

	-- Create global table
	_G.LeaMapsDB = _G.LeaMapsDB or {}

	-- Create local tables
	local LeaMapsLC, LeaMapsCB, LeaConfigList = {}, {}, {}

	-- Version
	LeaMapsLC["AddonVer"] = "8.3.04"
	LeaMapsLC["RestartReq"] = nil

	-- If client restart is required and has not been done, show warning and quit
	if LeaMapsLC["RestartReq"] then
		local metaVer = GetAddOnMetadata("Leatrix_Maps", "Version")
		if metaVer and metaVer ~= LeaMapsLC["AddonVer"] then
			C_Timer.After(1, function()
				print("NOTICE!|nYou must fully restart your game client before you can use this version of Leatrix Maps.")
			end)
			return
		end
	end

	-- Get locale table
	local void, Leatrix_Maps = ...
	local L = Leatrix_Maps.L

	----------------------------------------------------------------------
	-- L00: Leatrix Maps
	----------------------------------------------------------------------

	-- Main function
	function LeaMapsLC:MainFunc()

		-- Get player faction
		local playerFaction = UnitFactionGroup("player")

		-- Remove blackout frame
		WorldMapFrame.BlackoutFrame:SetAlpha(0)
		WorldMapFrame.BlackoutFrame:EnableMouse(false)

		-- Replace function to account for frame scale
		WorldMapFrame.ScrollContainer.GetCursorPosition = function(f)
			local x,y = MapCanvasScrollControllerMixin.GetCursorPosition(f)
			local s = WorldMapFrame:GetScale()
			return x/s, y/s
		end

		----------------------------------------------------------------------
		-- Show coordinates
		----------------------------------------------------------------------

		do

			-- Create background frame
			local cFrame = CreateFrame("FRAME", nil, WorldMapFrame.ScrollContainer)
			cFrame:SetSize(WorldMapFrame:GetWidth(), 17)
			cFrame:SetPoint("BOTTOMLEFT", 17)
			cFrame:SetPoint("BOTTOMRIGHT", 0)

			cFrame.t = cFrame:CreateTexture(nil, "BACKGROUND")
			cFrame.t:SetAllPoints()
			cFrame.t:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
			cFrame.t:SetVertexColor(0, 0, 0, 0.5)

			-- Create cursor coordinates frame
			local cCursor = CreateFrame("Frame", nil, WorldMapFrame.ScrollContainer)
			cCursor:SetSize(200, 16)
			cCursor:SetParent(cFrame)
			cCursor:ClearAllPoints()
			cCursor:SetPoint("BOTTOMLEFT", 152, 1)
			cCursor.x = cCursor:CreateFontString(nil, "ARTWORK", "GameFontNormal") 
			cCursor.x:SetJustifyH"LEFT"
			cCursor.x:SetAllPoints()
			cCursor.x:SetText(L["Cursor"] .. ": 88.8, 88.8")
			cCursor:SetWidth(cCursor.x:GetStringWidth() + 50)

			-- Create player coordinates frame
			local cPlayer = CreateFrame("Frame", nil, WorldMapFrame.ScrollContainer)
			cPlayer:SetSize(200, 16)
			cPlayer:SetParent(cFrame)
			cPlayer:ClearAllPoints()
			cPlayer:SetPoint("BOTTOMRIGHT", -132, 1)
			cPlayer.x = cPlayer:CreateFontString(nil, "ARTWORK", "GameFontNormal") 
			cPlayer.x:SetJustifyH"LEFT"
			cPlayer.x:SetAllPoints()
			cPlayer.x:SetText(L["Player"] .. ": 88.8, 88.8")
			cPlayer:SetWidth(cPlayer.x:GetStringWidth() + 50)

			-- Update timer
			local cPlayerTime = -1

			-- Update function
			cPlayer:SetScript("OnUpdate", function(self, elapsed)
				if cPlayerTime > 0.1 or cPlayerTime == -1 then
					-- Cursor coordinates
					local x, y = WorldMapFrame.ScrollContainer:GetNormalizedCursorPosition()
					if x and y and x > 0 and y > 0 and MouseIsOver(WorldMapFrame.ScrollContainer) then
						cCursor.x:SetFormattedText("%s: %.1f, %.1f", L["Cursor"], ((floor(x * 1000 + 0.5)) / 10), ((floor(y * 1000 + 0.5)) / 10))
					else
						cCursor.x:SetFormattedText("%s:", L["Cursor"])
					end
				end
				if cPlayerTime > 0.2 or cPlayerTime == -1 then
					-- Player coordinates
					local mapID = C_Map.GetBestMapForUnit("player")
					if not mapID then
						cPlayer.x:SetFormattedText("%s:", L["Player"])
						return
					end
					local position = C_Map.GetPlayerMapPosition(mapID,"player")
					if position and position.x ~= 0 and position.y ~= 0 then
						cPlayer.x:SetFormattedText("%s: %.1f, %.1f", L["Player"], position.x * 100, position.y * 100)
					else
						cPlayer.x:SetFormattedText("%s: %.1f, %.1f", L["Player"], 0, 0)
					end
					cPlayerTime = 0
				end
				cPlayerTime = cPlayerTime + elapsed
			end)

			-- Function to show or hide coordinates frames
			local function SetupCoords()
				if LeaMapsLC["ShowCoords"] == "On" then	cFrame:Show() else cFrame:Hide() end
			end

			LeaMapsCB["ShowCoords"]:HookScript("OnClick", SetupCoords)
			SetupCoords()

			-- Create configuration panel
			local cPanel = LeaMapsLC:CreatePanel("Coordinates", "cPanel")

			-- Add controls
			LeaMapsLC:MakeTx(cPanel, "Settings", 16, -72)
			LeaMapsLC:MakeCB(cPanel, "CoordsLargeFont", "Use large font", 16, -92, false, "If checked, coordinates will use a large font.")
			LeaMapsLC:MakeCB(cPanel, "CoordsBackground", "Show background", 16, -112, false, "If checked, coordinates will have a dark background texture.")

			-- Function to apply settings
			local function SetCoordFunc()
				if LeaMapsLC["CoordsLargeFont"] == "On" then
					cCursor.x:SetFont(cCursor.x:GetFont(), 16)
					cPlayer.x:SetFont(cPlayer.x:GetFont(), 16)
				else
					cCursor.x:SetFont(cCursor.x:GetFont(), 12)
					cPlayer.x:SetFont(cPlayer.x:GetFont(), 12)
				end
				if LeaMapsLC["CoordsBackground"] == "On" then
					cFrame.t:Show()
				else
					cFrame.t:Hide()
				end
			end

			-- Set coordinates settings when options are clicked and on startup
			LeaMapsCB["CoordsLargeFont"]:HookScript("OnClick", SetCoordFunc)
			LeaMapsCB["CoordsBackground"]:HookScript("OnClick", SetCoordFunc)
			SetCoordFunc()

			-- Back to Main Menu button click
			cPanel.b:HookScript("OnClick", function()
				cPanel:Hide()
				LeaMapsLC["PageF"]:Show()
			end)

			-- Reset button click
			cPanel.r:HookScript("OnClick", function()
				LeaMapsLC["CoordsLargeFont"] = "Off"
				LeaMapsLC["CoordsBackground"] = "On"
				SetCoordFunc()
				cPanel:Hide(); cPanel:Show()
			end)

			-- Show scale panel when configuration button is clicked
			LeaMapsCB["ShowCoordsBtn"]:HookScript("OnClick", function()
				if IsShiftKeyDown() and IsControlKeyDown() then
					-- Preset profile
					LeaMapsLC["CoordsLargeFont"] = "Off"
					LeaMapsLC["CoordsBackground"] = "On"
					SetCoordFunc()
					if cPanel:IsShown() then cPanel:Hide(); cPanel:Show(); end
				else
					cPanel:Show()
					LeaMapsLC["PageF"]:Hide()
				end
			end)

		end

		----------------------------------------------------------------------
		-- Remember zoom level
		----------------------------------------------------------------------

		do

			-- Set initial values
			local lastZoomLevel = WorldMapFrame.ScrollContainer:GetCanvasScale()
			local lastHorizontal = WorldMapFrame.ScrollContainer.targetScrollX
			local lastVertical = WorldMapFrame.ScrollContainer.targetScrollY
			local lastMapID = WorldMapFrame.mapID
			local lastMapSize = WorldMapFrame.isMaximized

			-- Function to save zoom level
			local function SaveZoomLevel()
				lastZoomLevel = WorldMapFrame.ScrollContainer:GetCanvasScale()
				lastHorizontal = WorldMapFrame.ScrollContainer.targetScrollX
				lastVertical = WorldMapFrame.ScrollContainer.targetScrollY
				lastMapID = WorldMapFrame.mapID
				lastMapSize = WorldMapFrame.isMaximized
			end

			-- Save zoom level when changed
			WorldMapFrame.ScrollContainer:HookScript("OnMouseUp", SaveZoomLevel)
			WorldMapFrame.ScrollContainer:HookScript("OnMouseWheel", SaveZoomLevel)

			-- Function to set zoom level
			local function SetZoomLevel()
				if LeaMapsLC["RememberZoom"] == "On" and WorldMapFrame:IsShown() then
					if WorldMapFrame.mapID == lastMapID and WorldMapFrame.isMaximized == lastMapSize then
						WorldMapFrame.ScrollContainer:InstantPanAndZoom(lastZoomLevel, 0.5, 0.5)
						WorldMapFrame.ScrollContainer:SetZoomTarget(lastZoomLevel)
						WorldMapFrame.ScrollContainer:SetPanTarget(lastHorizontal, lastVertical)
						WorldMapFrame.ScrollContainer:Hide(); WorldMapFrame.ScrollContainer:Show()
					end
				end
			end

			-- Set zoom level when map is shown
			hooksecurefunc("ToggleWorldMap", SetZoomLevel)
			hooksecurefunc("ToggleQuestLog", SetZoomLevel)
			hooksecurefunc(WorldMapFrame, "HandleUserActionToggleSidePanel", SetZoomLevel)

		end

		----------------------------------------------------------------------
		-- Unlock map frame
		----------------------------------------------------------------------

		if LeaMapsLC["UnlockMap"] == "On" then

			-- Create configuration panel
			local scaleFrame = LeaMapsLC:CreatePanel("Unlock Map", "scaleFrame")

			-- Add controls
			LeaMapsLC:MakeTx(scaleFrame, "Settings", 16, -72)
			LeaMapsLC:MakeCB(scaleFrame, "EnableMovement", "Allow frame movement", 16, -92, false, "If checked, you will be able to move the frame by dragging the border.")
			LeaMapsLC:MakeCB(scaleFrame, "StickyMapFrame", "Sticky map frame", 16, -112, false, "If checked, the map frame will remain open until you close it.")
			LeaMapsLC:MakeTx(scaleFrame, "Scale", 16, -152)
			LeaMapsLC:MakeSL(scaleFrame, "MapScale", "Windowed", "Drag to set the scale for the windowed map.", 0.5, 2, 0.05, 36, -192, "%.1f")
			LeaMapsLC:MakeSL(scaleFrame, "MaxMapScale", "Maximised", "Drag to set the scale for the maximised map.", 0.5, 2, 0.05, 206, -192, "%.1f")

			----------------------------------------------------------------------
			-- Allow map frame movement
			----------------------------------------------------------------------

			-- Remove frame management
			WorldMapFrame:SetAttribute("UIPanelLayout-area", nil)
			WorldMapFrame:SetAttribute("UIPanelLayout-enabled", false)
			WorldMapFrame:SetAttribute("UIPanelLayout-allowOtherPanels", true)

			-- Enable movement
			WorldMapFrame:SetMovable(true)
			WorldMapFrame:RegisterForDrag("LeftButton")
			WorldMapFrame:SetScript("OnDragStart", function()
				if LeaMapsLC["EnableMovement"] == "On" then
					WorldMapFrame:StartMoving()
				end
			end)
			WorldMapFrame:SetScript("OnDragStop", function()
				WorldMapFrame:StopMovingOrSizing()
				WorldMapFrame:SetUserPlaced(false)
				-- Save map frame position
				if WorldMapFrame:IsMaximized() then
					LeaMapsLC["MaxMapPosA"], void, LeaMapsLC["MaxMapPosR"], LeaMapsLC["MaxMapPosX"], LeaMapsLC["MaxMapPosY"] = WorldMapFrame:GetPoint()
				else
					LeaMapsLC["MapPosA"], void, LeaMapsLC["MapPosR"], LeaMapsLC["MapPosX"], LeaMapsLC["MapPosY"] = WorldMapFrame:GetPoint()
				end
			end)

			-- Set position when map size is toggled
			hooksecurefunc(WorldMapFrame, "SynchronizeDisplayState", function()
				WorldMapFrame:ClearAllPoints()
				if not WorldMapFrame:IsMaximized() then
					WorldMapFrame:SetPoint(LeaMapsLC["MapPosA"], UIParent, LeaMapsLC["MapPosR"], LeaMapsLC["MapPosX"], LeaMapsLC["MapPosY"])
				else
					WorldMapFrame:SetPoint(LeaMapsLC["MaxMapPosA"], UIParent, LeaMapsLC["MaxMapPosR"], LeaMapsLC["MaxMapPosX"], LeaMapsLC["MaxMapPosY"])
				end
			end)

			-- Set position on startup
			WorldMapFrame:ClearAllPoints()
			if not WorldMapFrame:IsMaximized() then
				WorldMapFrame:SetPoint(LeaMapsLC["MapPosA"], UIParent, LeaMapsLC["MapPosR"], LeaMapsLC["MapPosX"], LeaMapsLC["MapPosY"])
			else
				WorldMapFrame:SetPoint(LeaMapsLC["MaxMapPosA"], UIParent, LeaMapsLC["MaxMapPosR"], LeaMapsLC["MaxMapPosX"], LeaMapsLC["MaxMapPosY"])
			end

			----------------------------------------------------------------------
			-- Map scale
			----------------------------------------------------------------------

			-- Function to set map frame scale
			local function SetMapScale()
				LeaMapsCB["MapScale"].f:SetFormattedText("%.0f%%", LeaMapsLC["MapScale"] * 100)
				LeaMapsCB["MaxMapScale"].f:SetFormattedText("%.0f%%", LeaMapsLC["MaxMapScale"] * 100)
				if not WorldMapFrame:IsMaximized() then
					WorldMapFrame:SetScale(LeaMapsLC["MapScale"])
				else
					WorldMapFrame:SetScale(LeaMapsLC["MaxMapScale"])
				end
			end

			-- Set scale properties when controls are changed and on startup
			LeaMapsCB["MapScale"]:HookScript("OnValueChanged", SetMapScale)
			LeaMapsCB["MaxMapScale"]:HookScript("OnValueChanged", SetMapScale)
			SetMapScale()

			-- Set scale when map size is toggled
			hooksecurefunc(WorldMapFrame, "SynchronizeDisplayState", SetMapScale)

			----------------------------------------------------------------------
			-- Sticky map frame
			----------------------------------------------------------------------

			-- Function to set sticky map frame mode
			local function StickyMapFunc()
				if LeaMapsLC["StickyMapFrame"] == "On" then
					for k, v in pairs(UISpecialFrames) do
						if v == "WorldMapFrame" then
							table.remove(UISpecialFrames, k)
						end
					end
				else
					if not tContains(UISpecialFrames, "WorldMapFrame") then
						table.insert(UISpecialFrames, "WorldMapFrame")
					end
				end
			end

			-- Set sticky map frame mode when option is clicked and on startup
			LeaMapsCB["StickyMapFrame"]:HookScript("OnClick", StickyMapFunc)
			StickyMapFunc()

			----------------------------------------------------------------------
			-- Panel button handlers
			----------------------------------------------------------------------

			-- Back to Main Menu button click
			scaleFrame.b:HookScript("OnClick", function()
				scaleFrame:Hide()
				LeaMapsLC["PageF"]:Show()
			end)

			-- Reset button click
			scaleFrame.r:HookScript("OnClick", function()
				-- Reset map position
				LeaMapsLC["EnableMovement"] = "On"
				LeaMapsLC["MapPosA"], LeaMapsLC["MapPosR"], LeaMapsLC["MapPosX"], LeaMapsLC["MapPosY"] = "TOPLEFT", "TOPLEFT", 16, -94
				LeaMapsLC["MaxMapPosA"], LeaMapsLC["MaxMapPosR"], LeaMapsLC["MaxMapPosX"], LeaMapsLC["MaxMapPosY"] = "CENTER", "CENTER", 0, 0
				WorldMapFrame:ClearAllPoints()
				if WorldMapFrame:IsMaximized() then
					WorldMapFrame:SetPoint(LeaMapsLC["MaxMapPosA"], UIParent, LeaMapsLC["MaxMapPosR"], LeaMapsLC["MaxMapPosX"], LeaMapsLC["MaxMapPosY"])
				else
					WorldMapFrame:SetPoint(LeaMapsLC["MapPosA"], UIParent, LeaMapsLC["MapPosR"], LeaMapsLC["MapPosX"], LeaMapsLC["MapPosY"])
				end
				-- Reset map scale
				LeaMapsLC["MapScale"] = 1.0
				LeaMapsLC["MaxMapScale"] = 0.9
				SetMapScale()
				-- Reset sticky map frame
				LeaMapsLC["StickyMapFrame"] = "Off"
				StickyMapFunc()
				-- Refresh panel
				scaleFrame:Hide(); scaleFrame:Show()
			end)

			-- Show scale panel when configuration button is clicked
			LeaMapsCB["UnlockMapBtn"]:HookScript("OnClick", function()
				if IsShiftKeyDown() and IsControlKeyDown() then
					-- Preset profile
					LeaMapsLC["EnableMovement"] = "On"
					LeaMapsLC["MapScale"] = 1.0
					LeaMapsLC["MaxMapScale"] = 0.9
					SetMapScale()
					LeaMapsLC["StickyMapFrame"] = "Off"
					StickyMapFunc()
					if scaleFrame:IsShown() then scaleFrame:Hide(); scaleFrame:Show(); end
				else
					scaleFrame:Show()
					LeaMapsLC["PageF"]:Hide()
				end
			end)

		else

			-- Unlock map is disabled so set maximised world map position and scale on startup and when map size is toggled
			hooksecurefunc(WorldMapFrame, "SynchronizeDisplayState", function()
				if WorldMapFrame:IsMaximized() then
					-- Lower fullscreen map as default position is too high
					WorldMapFrame:ClearAllPoints()
					WorldMapFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
					WorldMapFrame:SetScale(0.9)
				else
					WorldMapFrame:SetScale(1.0)
				end
			end)

		end

		----------------------------------------------------------------------
		-- Disable map fade while moving
		----------------------------------------------------------------------

		-- Function to set map fade
		local function SetMapFade()
			if LeaMapsLC["NoMapFade"] == "On" then
				SetCVar("mapFade", "0")
			else
				SetCVar("mapFade", "1")
			end
		end

		-- Set map fade when option is clicked and on startup
		LeaMapsCB["NoMapFade"]:HookScript("OnClick", SetMapFade)
		SetMapFade()

		----------------------------------------------------------------------
		-- Disable map emote when opening the map
		----------------------------------------------------------------------

		hooksecurefunc("DoEmote", function(emote)
			if emote == "READ" and WorldMapFrame:IsShown() then
				if LeaMapsLC["NoMapEmote"] == "On" then
					CancelEmote()
				end
			end
		end)

		----------------------------------------------------------------------
		-- Show dungeon location icons (/ltp pos)
		----------------------------------------------------------------------

		do

			-- Disable integrated dungeon icons
			SetCVar("showDungeonEntrancesOnMap", "0")
			local cFrame = CreateFrame("FRAME")
			cFrame:RegisterEvent("CVAR_UPDATE")
			cFrame:SetScript("OnEvent", function(self, event, cset)
				if cset == "SHOW_DUNGEON_ENTRANCES" and GetCVar("showDungeonEntrancesOnMap") ~= "0" then
					if not LeaMapsLC["DebugMode"] then
						SetCVar("showDungeonEntrancesOnMap", "0")
					end
				end
			end)

			local dnTex, rdTex = "Dungeon", "Raid"
			local pATex, pHTex, pNTex = "TaxiNode_Continent_Alliance", "TaxiNode_Continent_Horde", "TaxiNode_Continent_Neutral"
			local chTex = "ChallengeMode-icon-chest"

			local PinData = {

				-- World Of Warcraft: Eastern Kingdoms
				[27] =  --[[Dun Morogh]] {{31.1, 37.9, L["Gnomeregan"], L["Dungeon"], dnTex},},
				[15] =  --[[Badlands]] {{41.7, 11.6, L["Uldaman"], L["Dungeon"], dnTex},},
				[16] =  --[[Uldaman]] {{36.7, 29.4, L["Uldaman"], L["Dungeon"], dnTex},},
				[18] =  --[[Tirisfal Glades]] {{82.5, 33.3, L["Scarlet Halls"] .. ", " .. L["Scarlet Monastery"], L["Dungeon"], dnTex},},
				[19] =  --[[Scarlet Monastery]] {{69.2, 24.4, L["Scarlet Monastery"], L["Dungeon"], dnTex}, {78.6, 58.9, L["Scarlet Halls"], L["Dungeon"], dnTex},},
				[21] =  --[[Silverpine Forest]] {{44.8, 67.8, L["Shadowfang Keep"], L["Dungeon"], dnTex},},
				[22] =  --[[Western Plaguelands]] {{69.6, 73.2, L["Scholomance"], L["Dungeon"], dnTex},},
				[23] =  --[[Eastern Plaguelands]] {{27.7, 11.6, L["Stratholme: Crusader's Square"], L["Dungeon"], dnTex}, {43.5, 19.4, L["Stratholme: The Gauntlet"], L["Dungeon"], dnTex},},
				[30] =  --[[New Tinkertown]] {{30.2, 74.7, L["Gnomeregan"], L["Dungeon"], dnTex},},
				[32] =  --[[Searing Gorge]] {{34.9, 83.9, L["Blackrock Mountain"], L["Blackrock Caverns"] .. "," .. L["Blackrock Depths"] .. "|n" .. L["Blackrock Spire"] .. "," .. L["Blackwing Lair"] .. "," .. L["Molten Core"], dnTex},},
				[33] =  --[[Blackrock Mountain]] {{66.5, 60.7, L["Blackrock Caverns"], L["Dungeon"], dnTex}, {64.3, 70.9, L["Blackwing Lair"], L["Raid"], rdTex}, {65.9, 41.9, L["Blackrock Spire"], L["Dungeon"], dnTex}, {80.3, 40.7, L["Lower Blackrock Spire"], L["Dungeon"], dnTex}, {79.0, 33.7, L["Upper Blackrock Spire"], L["Dungeon"], dnTex},},
				[34] =  --[[Blackrock Mountain]] {{71.9, 53.5, L["Blackrock Caverns"], L["Dungeon"], dnTex},},
				[35] =  --[[Blackrock Mountain]] {{54.3, 83.4, L["Molten Core"], L["Raid"], rdTex}, {39.0, 18.3, L["Blackrock Depths"], L["Dungeon"], dnTex},},
				[36] =  --[[Burning Steppes]] {{21.0, 37.9, L["Blackrock Mountain"], L["Blackrock Caverns"] .. "," .. L["Blackrock Depths"] .. "|n" .. L["Blackrock Spire"] .. "," .. L["Blackwing Lair"] .. "," .. L["Molten Core"], dnTex}, {23.0, 26.3, L["Blackwing Descent"], L["Raid"], rdTex},},
				[37] =  --[[Elwynn Forest]] {{19.1, 36.9, L["The Stockade"], L["Dungeon"], dnTex},},
				[42] =  --[[Deadwind Pass]] {{46.9, 74.7, L["Karazhan"], L["Raid"], rdTex}, {46.7, 70.2, L["Return to Karazhan"], L["Dungeon"], dnTex},},
				[50] =  --[[Northern Stranglethorn]] {{72.1, 32.9, L["Zul'Gurub"], L["Dungeon"], dnTex},},
				[51] =  --[[Swamp of Sorrows]] {{69.7, 53.5, L["Temple of Atal'Hakkar"], L["Dungeon"], dnTex},},
				[52] =  --[[Westfall]] {{42.6, 71.8, L["The Deadmines"], L["Dungeon"], dnTex},},
				[55] =  --[[The Deadmines]] {{25.5, 51.1, L["The Deadmines"], L["Dungeon"], dnTex},},
				[84] =  --[[Stormwind City]] {{49.5, 69.5, L["The Stockade"], L["Dungeon"], dnTex},},
				[224] =  --[[Stranglethorn Vale]] {{63.5, 21.6, L["Zul'Gurub"], L["Dungeon"], dnTex},},
				[243] =  --[[Blackrock Depths]] {{68.4, 38.3, L["Molten Core"], L["Raid"], rdTex},},
				[469] =  --[[New Tinkertown]] {{32.6, 37.0, L["Gnomeregan"], L["Dungeon"], dnTex},},

				-- World Of Warcraft: Kalimdor
				[10] =  --[[Northern Barrens]] {{38.9, 69.1, L["Wailing Caverns"], L["Dungeon"], dnTex},},
				[11] =  --[[Wailing Caverns]] {{55.1, 65.9, L["Wailing Caverns"], L["Dungeon"], dnTex},},
				[63] =  --[[Ashenvale]] {{14.2, 13.9, L["Blackfathom Deeps"], L["Dungeon"], dnTex},},
				[64] =  --[[Thousand Needles]] {{41.5, 29.4, L["Razorfen Downs"], L["Dungeon"], dnTex},},
				[66] =  --[[Desolace]] {{29.1, 62.6, L["Maraudon"], L["Dungeon"], dnTex},},
				[67] =  --[[Maraudon]] {{78.2, 56.0, L["Maraudon: Foulspore Cavern"], L["Dungeon"], dnTex},},
				[68] =  --[[Maraudon]] {{44.4, 76.8, L["Maraudon: Earth Song Falls"], L["Dungeon"], dnTex}, {52.0, 24.5, L["Maraudon: The Wicked Grotto"], L["Dungeon"], dnTex},},
				[69] =  --[[Feralas]] {{60.3, 31.3, L["Dire Maul: Capital Gardens"], L["Dungeon"], dnTex}, {64.8, 30.2, L["Dire Maul: Warpwood Quarter"], L["Dungeon"], dnTex}, {62.5, 24.9, L["Dire Maul: Gordok Commons"], L["Dungeon"], dnTex},},
				[70] =  --[[Dustwallow Marsh]] {{52.2, 75.7, L["Onyxia's Lair"], L["Raid"], rdTex},},
				[71] =  --[[Tanaris]] {{39.2, 21.3, L["Zul'Farrak"], L["Dungeon"], dnTex}, {64.8, 50.0, L["Caverns of Time"], L["Black Morass"] .. ", " .. L["Culling of Stratholme"] .. ",|n" .. L["Dragon Soul"] .. ", " .. L["End Time"] .. ", " .. L["Hour of Twilight"] .. ",|n" .. L["Hyjal Summit"] .. ", " .. L["Old Hillsbrad Foothills"] .. ",|n" .. L["Well of Eternity"], dnTex},},
				[75] =  --[[Caverns of Time]] {{57.5, 82.6, L["The Culling of Stratholme"], L["Dungeon"], dnTex}, {36.7, 83.0, L["The Black Morass"], L["Dungeon"], dnTex}, {22.5, 64.4, L["Well of Eternity"], L["Dungeon"], dnTex}, {26.9, 35.8, L["Old Hillsbrad Foothills"], L["Dungeon"], dnTex}, {35.5, 15.6, L["Hyjal Summit"], L["Raid"], rdTex}, {57.3, 29.6, L["End Time"], L["Dungeon"], dnTex}, {61.6, 26.6, L["Dragon Soul"], L["Raid"], rdTex}, {66.9, 29.4, L["Hour of Twilight"], L["Dungeon"], dnTex},},
				[85] =  --[[Orgrimmar]] {{55.2, 51.2, L["Ragefire Chasm"], L["Dungeon"], dnTex},},
				[86] =  --[[Orgrimmar]] {{70.0, 49.2, L["Ragefire Chasm"], L["Dungeon"], dnTex},},
				[199] =  --[[Southern Barrens]] {{41.0, 94.6, L["Razorfen Kraul"], L["Dungeon"], dnTex},},
				[247] =  --[[Ruins of Ahn'Qiraj]] {{59.3, 28.7, L["Scarab Coffer"], L["Chest"], chTex}, {60.8, 51.0, L["Scarab Coffer"], L["Chest"], chTex}, {73.0, 66.4, L["Scarab Coffer"], L["Chest"], chTex},  {57.4, 78.3, L["Scarab Coffer"], L["Chest"], chTex}, {54.8, 87.5, L["Scarab Coffer"], L["Chest"], chTex}, {41.0, 76.9, L["Scarab Coffer"], L["Chest"], chTex}, {34.0, 53.0, L["Scarab Coffer"], L["Chest"], chTex}, {41.1, 32.2, L["Scarab Coffer"], L["Chest"], chTex}, {41.6, 46.3, L["Scarab Coffer"], L["Chest"], chTex}, {46.7, 42.0, L["Scarab Coffer"], L["Chest"], chTex},}, 
				[319] =  --[[Temple of Ahn'Qiraj]] {{33.1, 48.4, L["Large Scarab Coffer"], L["Chest"], chTex}, {64.5, 25.5, L["Large Scarab Coffer"], L["Chest"], chTex}, {58.4, 49.9, L["Large Scarab Coffer"], L["Chest"], chTex}, {47.5, 54.7, L["Large Scarab Coffer"], L["Chest"], chTex}, {56.2, 66.0, L["Large Scarab Coffer"], L["Chest"], chTex}, {50.7, 78.1, L["Large Scarab Coffer"], L["Chest"], chTex}, {51.4, 83.2, L["Large Scarab Coffer"], L["Chest"], chTex}, {48.4, 85.4, L["Large Scarab Coffer"], L["Chest"], chTex}, {48.0, 81.1, L["Large Scarab Coffer"], L["Chest"], chTex}, {34.2, 83.5, L["Large Scarab Coffer"], L["Chest"], chTex}, {39.2, 68.4, L["Large Scarab Coffer"], L["Chest"], chTex},}, 
				[327] =  --[[Ahn'Qiraj: The Fallen Kingdom]] {{46.8, 7.5, L["Temple of Ahn'Qiraj"], L["Raid"], rdTex}, {58.9, 14.3, L["Ruins of Ahn'Qiraj"], L["Raid"], rdTex},},

				-- The Burning Crusade
				[95] =  --[[Ghostlands]] {{82.1, 64.3, L["Zul'Aman"], L["Dungeon"], dnTex},},
				[100] =  --[[Hellfire Peninsula]] {{46.6, 52.8, L["Magtheridon's Lair"], L["Raid"], rdTex}, {47.7, 53.6, L["Hellfire Ramparts"], L["Dungeon"], dnTex}, {47.7, 52.0, L["The Shattered Halls"], L["Dungeon"], dnTex}, {46.0, 51.8, L["The Blood Furnace"], L["Dungeon"], dnTex},},
				[102] =  --[[Zangarmarsh]] {{50.4, 40.9, L["Coilfang Reservoir"], L["Serpentshrine Cavern"] .. ", " .. L["Slave Pens"] .. ",|n" .. L["Steamvault"] .. ", " .. L["Underbog"], dnTex},},
				[103] =  --[[The Exodar]] {{48.3, 62.9, L["Stormwind"], L["Portal"], pATex},},
				[104] =  --[[Shadowmoon Valley]] {{71.0, 46.4, L["Black Temple"], L["Raid"], rdTex},},
				[105] =  --[[Blade's Edge Mountains]] {{68.7, 24.3, L["Gruul's Lair"], L["Raid"], rdTex},},
				[108] =  --[[Terokkar Forest]] {{43.2, 65.6, L["Sethekk Halls"], L["Dungeon"], dnTex}, {36.1, 65.6, L["Auchenai Crypts"], L["Dungeon"], dnTex}, {39.6, 71.0, L["Shadow Labyrinth"], L["Dungeon"], dnTex}, {39.7, 60.2, L["Mana-Tombs"], L["Dungeon"], dnTex},},
				[109] =  --[[Netherstorm]] {{70.6, 69.7, L["The Mechanar"], L["Dungeon"], dnTex}, {73.7, 63.7, L["The Eye"], L["Raid"], rdTex}, {71.7, 55.0, L["The Botanica"], L["Dungeon"], dnTex}, {74.4, 57.7, L["The Arcatraz"], L["Dungeon"], dnTex},},
				[110] =  --[[Silvermoon City]] {{58.5, 18.7, L["Orgrimmar"], L["Portal"], pHTex},},
				[122] =  --[[Isle of Quel'Danas]] {{61.2, 30.9, L["Magisters' Terrace"], L["Dungeon"], dnTex}, {44.3, 45.6, L["Sunwell Plateau"], L["Raid"], rdTex},},

				-- Wrath Of The Lich King
				[114] =  --[[Borean Tundra]] {{27.6, 26.6, L["The Nexus"], L["The Nexus"] .. ", " .. L["The Oculus"] .. ",|n" .. L["The Eye of Eternity"], dnTex},},
				[115] =  --[[Dragonblight]] {{59.6, 51.1, L["Wyrmrest Temple"], L["The Ruby Sanctum"] .. ", " .. L["The Obsidian Sanctum"], dnTex}, {87.4, 51.1, L["Naxxramas"], L["Raid"], rdTex}, {26.2, 49.6, L["Azjol-Nerub"], L["Azjol-Nerub"] .. ", " .. L["The Old Kingdom"], dnTex},},
				[116] =  --[[Grizzly Hills]] {{17.5, 27.0, L["Drak'Tharon Keep"], L["Dungeon"], dnTex},},
				[117] =  --[[Howling Fjord]] {{57.3, 46.8, L["Utgarde Keep"], L["Utgarde Keep"] .. ", " .. L["Utgarde Pinnacle"], dnTex},},
				[118] =  --[[Icecrown]] {{53.3, 85.5, L["Icecrown Citadel"], L["Raid"], rdTex}, {52.6, 89.4, L["The Frozen Halls"], L["The Forge of Souls"] .. ", " .. L["The Pit of Saron"] .. ",|n" .. L["The Halls of Reflection"], dnTex}, {74.2, 20.5, L["Trial of the Champion"], L["Dungeon"], dnTex}, {75.1, 21.8, L["Trial of the Crusader"], L["Raid"], rdTex},},
				[120] =  --[[The Storm Peaks]] {{39.6, 26.9, L["Halls of Stone"], L["Dungeon"], dnTex}, {45.4, 21.4, L["Halls of Lightning"], L["Dungeon"], dnTex}, {41.6, 17.8, L["Ulduar"], L["Raid"], rdTex},},
				[121] =  --[[Zul'Drak]] {{29.0, 83.9, L["Drak'Tharon Keep"], L["Dungeon"], dnTex}, {76.2, 21.1, L["Gundrak"], L["Dungeon"], dnTex}, {81.2, 28.9, L["Gundrak (rear entrance)"], L["Dungeon"], dnTex},},
				[123] =  --[[Wintergrasp]] {{50.5, 16.4, L["Vault of Archavon"], L["Raid"], rdTex},},
				[125] =  --[[Dalaran]] {{66.8, 68.2, L["The Violet Hold"], L["Dungeon"], dnTex},},

				-- Cataclysm
				[198] =  --[[Mount Hyjal]] {{47.3, 78.0, L["Firelands"], L["Raid"], rdTex},},
				[203] =  --[[Vashj'ir]] {{49.1, 42.4, L["Throne of the Tides"], L["Dungeon"], dnTex},},
				[204] =  --[[Abyssal Depths]] {{70.8, 29.0, L["Throne of the Tides"], L["Dungeon"], dnTex},},
				[207] =  --[[Deepholm]] {{47.6, 52.0, L["The Stonecore"], L["Dungeon"], dnTex},},
				[241] =  --[[Twilight Highlands]] {{34.0, 78.0, L["The Bastion of Twilight"], L["Raid"], rdTex}, {19.2, 54.0, L["Grim Batol"], L["Dungeon"], dnTex},},
				[244] =  --[[Tol Barad]] {{46.3, 47.9, L["Baradin Hold"], L["Raid"], rdTex},},
				[249] =  --[[Uldum]] {{71.6, 52.2, L["Halls of Origination"], L["Dungeon"], dnTex}, {60.5, 64.2, L["Lost City of the Tol'vir"], L["Dungeon"], dnTex}, {76.7, 84.4, L["The Vortex Pinnacle"], L["Dungeon"], dnTex}, {38.4, 80.6, L["Throne of the Four Winds"], L["Raid"], rdTex},},

				-- Mists Of Pandaria
				[371] =  --[[The Jade Forest]] {{56.2, 57.9, L["Temple of the Jade Serpent"], L["Dungeon"], dnTex},},
				[376] =  --[[Valley of the Four Winds]] {{36.1, 69.2, L["Stormstout Brewery"], L["Dungeon"], dnTex},},
				[379] =  --[[Kun-Lai Summit]] {{59.6, 39.2, L["Mogu'shan Vaults"], L["Raid"], rdTex}, {36.7, 47.4, L["Shado-Pan Monastery"], L["Dungeon"], dnTex},},
				[388] =  --[[Townlong Steppes]] {{34.7, 81.5, L["Siege of Niuzao Temple"], L["Dungeon"], dnTex}, {49.7, 68.7, L["Throne of Thunder"], L["Raid"], rdTex},},
				[390] =  --[[Vale of Eternal Blossoms]] {{80.9, 32.7, L["Mogu'shan Palace"], L["Dungeon"], dnTex}, {72.4, 44.2, L["Siege of Orgrimmar"], L["Raid"], rdTex}, {15.8, 74.3, L["Gate of the Setting Sun"], L["Dungeon"], dnTex},},
				[422] =  --[[Dread Wastes]] {{38.8, 35.0, L["Heart of Fear"], L["Raid"], rdTex},},
				[433] =  --[[The Veiled Stair]] {{48.4, 61.4, L["Terrace of Endless Spring"], L["Raid"], rdTex},},
				[504] =  --[[Isle of Thunder]] {{63.6, 32.3, L["Throne of Thunder"], L["Raid"], rdTex},},

				-- Warlords Of Draenor
				[525] =  --[[Frostfire Ridge]] {{49.8, 24.7, L["Bloodmaul Slag Mines"], L["Dungeon"], dnTex},},
				[534] =  --[[Tanaan Jungle]] {{45.7, 53.5, L["Hellfire Citadel"], L["Raid"], rdTex},},
				[535] =  --[[Talador]] {{46.3, 73.9, L["Auchindoun"], L["Dungeon"], dnTex},},
				[539] =  --[[Shadowmoon Valley]] {{31.9, 42.5, L["Shadowmoon Burial Grounds"], L["Dungeon"], dnTex},},
				[542] =  --[[Spires of Arak]] {{35.6, 33.6, L["Skyreach"], L["Dungeon"], dnTex},},
				[543] =  --[[Gorgrond]] {{51.3, 28.6, L["Blackrock Foundry"], L["Raid"], rdTex}, {55.2, 31.9, L["Grimrail Depot"], L["Dungeon"], dnTex}, {59.6, 45.5, L["The Everbloom"], L["Dungeon"], dnTex}, {45.4, 13.5, L["Iron Docks"], L["Dungeon"], dnTex},},
				[550] =  --[[Nagrand]] {{32.9, 38.4, L["Highmaul"], L["Raid"], rdTex},},
				[622] =  --[[Stormshield]] {{60.8, 38.0, L["Stormwind"], L["Portal"], pATex}, {36.4, 41.1, L["Lion's Watch"], L["Portal"], pATex, 38445},},
				[624] =  --[[Warspear]] {{60.6, 51.6, L["Orgrimmar"], L["Portal"], pHTex}, {53.0, 43.9, L["Vol'mar"], L["Portal"], pHTex, 37935},},

				-- Legion
				[885] =  --[[Antoran Wastes]] {{55.0, 62.4, L["Antorus, the Burning Throne"], L["Raid"], rdTex},},
				[630] =  --[[Azsuna]] {{61.1, 41.1, L["Eye of Azshara"], L["Dungeon"], dnTex}, {48.1, 82.1, L["Vault of the Wardens"], L["Dungeon"], dnTex},},
				[646] =  --[[Broken Shore]] {{64.7, 16.6, L["Cathedral of Eternal Night"], L["Dungeon"], dnTex}, {64.1, 21.3, L["Tomb of Sargeras"], L["Raid"], rdTex},},
				[627] =  --[[Dalaran]] {{66.5, 68.6, L["The Violet Hold"], L["Dungeon"], dnTex},},
				[682] =  --[[Felsoul Hold]] {{53.6, 36.8, L["Shal'Aran"], L["Portal"], pNTex, 41575,},},
				[650] =  --[[Highmountain]] {{49.6, 68.7, L["Neltharion's Lair"], L["Dungeon"], dnTex},},
				[882] =  --[[Mac'Aree]] {{22.2, 56.2, L["Seat of the Triumvirate"], L["Dungeon"], dnTex},},
				[684] =  --[[Shattered Locus]] {{40.9, 13.7, L["Shal'Aran"], L["Portal"], pNTex, 42230,},},
				[634] =  --[[Stormheim]] {{70.4, 69.5, L["Gates of Valor"], L["Halls of Valor"] .. "," .. L["Trials of Valor"], dnTex}, {52.5, 45.3, L["Maw of Souls"], L["Dungeon"], dnTex},},
				[680] =  --[[Suramar]] {{21.6, 28.5, L["Falanaar"], L["Portal"], pNTex, 42230,}, {39.7, 76.2, L["Felsoul Hold"], L["Portal"], pNTex, 41575,}, {30.8, 11.0, L["Moon Guard Stronghold"], L["Portal"], pNTex, 43808,}, {43.7, 79.2, L["Lunastre Estate"], L["Portal"], pNTex, 43811,}, {36.1, 47.2, L["Ruins of Elune'eth"], L["Portal"], pNTex, 40956,}, {52.0, 78.8, L["Evermoon Terrace"], L["Portal"], pNTex, 42889,}, {43.4, 60.6, L["Sanctum of Order"], L["Portal"], pNTex, 43813,}, {42.0, 35.2, L["Tel'anor"], L["Portal"], pNTex, 43809,}, {64.0, 60.4, L["Twilight Vineyards"], L["Portal"], pNTex, 44084,}, {54.5, 69.4, L["Astravar Harbor"], L["Portal"], pNTex, 44740,}, {47.7, 81.4, L["The Waning Crescent"], L["Portal"], pNTex, 42487, 38649,}, {43.3, 62.3, L["Sanctum of Order"], L["The Arcway"] .. "," .. L["The Nighthold"], dnTex}, {50.9, 65.6, L["Court of Stars"], L["Dungeon"], dnTex},},
				[641] =  --[[Val'sharah]] {{38.6, 51.0, L["Black Rook Hold"], L["Dungeon"], dnTex}, {59.0, 31.1, L["Darkheart Thicket"], L["Dungeon"], dnTex}, {56.3, 36.9, L["The Emerald Nightmare"], L["Raid"], rdTex},},

				-- Battle For Azeroth
				[896] =  --[[Drustvar]] {{33.7, 12.6, L["Waycrest Manor"], L["Dungeon"], dnTex},},
				[1462]=  --[[Mechagon Island]] {{73.2, 36.3, L["Operation: Mechagon"], L["Dungeon"], dnTex},},
				[863] =  --[[Nazmir]] {{51.4, 64.9, L["The Underrot"], L["Dungeon"], dnTex}, {54.1, 63.0, L["Uldir"], L["Raid"], rdTex},},
				[942] =  --[[Stormsong Valley]] {{83.9, 46.9, L["Crucible of Storms"], L["Raid"], rdTex}, {78.9, 26.5, L["Shrine of the Storm"], L["Dungeon"], dnTex},},
				[895] =  --[[Tiragarde Sound]] {{84.5, 78.8, L["Freehold"], L["Dungeon"], dnTex},},
				[864] =  --[[Vol'dun]] {{51.9, 25.1, L["Temple of Sethraliss"], L["Dungeon"], dnTex},},
				[862] =  --[[Zuldazar]] {{43.4, 39.5, L["Atal'Dazar"], L["Dungeon"], dnTex}, {37.6, 39.5, L["Kings' Rest"], L["Dungeon"], dnTex},},
				[1527] =  --[[Uldum (N'Zoth)]] {{55.1, 43.9, L["Ny'alotha, The Waking City"], L["Raid"] .. " (" .. L["available during zone attack"] .. ")", rdTex}, {71.6, 52.2, L["Halls of Origination"], L["Dungeon"], dnTex}, {60.5, 64.2, L["Lost City of the Tol'vir"], L["Dungeon"], dnTex}, {76.7, 84.4, L["The Vortex Pinnacle"], L["Dungeon"], dnTex}, {38.4, 80.6, L["Throne of the Four Winds"], L["Raid"], rdTex},},
				[1530] =  --[[Vale of Eternal Blossoms (N'Zoth)]] {{40.1, 45.5, L["Ny'alotha, The Waking City"], L["Raid"] .. " (" .. L["available during zone attack"] .. ")", rdTex}, {81.4, 30.2, L["Mogu'shan Palace"], L["Dungeon"], dnTex}, {72.6, 42.0, L["Siege of Orgrimmar"], L["Raid"], rdTex}, {15.5, 72.2, L["Gate of the Setting Sun"], L["Dungeon"], dnTex},},

			}

			-- Faction specific dungeon and raid entrances
			if playerFaction == "Alliance" then
				--[[Boralus]] 		PinData[1161] = PinData[1161] or {}; 	tinsert(PinData[1161], {70.5, 35.4, L["Battle of Dazar'alor"], L["Raid"], rdTex, nil, nil, nil, nil})
				--[[Boralus]] 		PinData[1161] = PinData[1161] or {}; 	tinsert(PinData[1161], {72.4, 19.0, L["Siege of Boralus"], L["Dungeon"], dnTex, nil, nil, nil, nil})
				--[[Tiragarde Snd]]	PinData[895] = PinData[895] or {}; 		tinsert(PinData[895], {74.4, 28.2, L["Battle of Dazar'alor"], L["Raid"], rdTex, nil, nil, nil, nil})
				--[[Tiragarde Snd]]	PinData[895] = PinData[895] or {}; 		tinsert(PinData[895], {74.8, 24.1, L["Siege of Boralus"], L["Dungeon"], dnTex, nil, nil, nil, nil})
				--[[Zuldazar]] 		PinData[862] = PinData[862] or {}; 		tinsert(PinData[862], {39.2, 71.5, L["The MOTHERLODE!!"], L["Dungeon"], dnTex, nil, nil, nil, nil})

			elseif playerFaction == "Horde" then
				--[[Dazar'alor]] 	PinData[1165] = PinData[1165] or {}; 	tinsert(PinData[1165], {38.9, 2.7, L["Battle of Dazar'alor"], L["Raid"], rdTex, nil, nil, nil, nil})
				--[[Dazar'alor]] 	PinData[1165] = PinData[1165] or {}; 	tinsert(PinData[1165], {44.3, 92.7, L["The MOTHERLODE!!"], L["Dungeon"], dnTex, nil, nil, nil, nil})
				--[[Tiragarde Snd]]	PinData[895] = PinData[895] or {}; 		tinsert(PinData[895], {88.3, 51.0, L["Siege of Boralus"], L["Dungeon"], dnTex, nil, nil, nil, nil})
				--[[Zuldazar]] 		PinData[862] = PinData[862] or {}; 		tinsert(PinData[862], {54.3, 29.9, L["Battle of Dazar'alor"], L["Raid"], rdTex, nil, nil, nil, nil})
				--[[Zuldazar]] 		PinData[862] = PinData[862] or {}; 		tinsert(PinData[862], {56.1, 59.9, L["The MOTHERLODE!!"], L["Dungeon"], dnTex, nil, nil, nil, nil})

			end

			-- Add Caverns of Time portal to Shattrath if reputation with Keepers of Time is revered or higher
			local name, description, standingID = GetFactionInfoByID(989) 
			if standingID and standingID >= 7 then
				PinData[111] = --[[Shattrath City]] {{74.7, 31.4, L["Caverns of Time"], L["Portal from Zephyr"], pNTex},}
			end

			local LeaMix = CreateFromMixins(MapCanvasDataProviderMixin)

			function LeaMix:RefreshAllData()

				-- Disable integrated dungeon icons
				if GetCVar("showDungeonEntrancesOnMap") ~= "0" then
					if not LeaMapsLC["DebugMode"] then
						SetCVar("showDungeonEntrancesOnMap", "0")
					end
				end

				-- Remove all pins created by Leatrix Maps
				self:GetMap():RemoveAllPinsByTemplate("LeaMapsGlobalPinTemplate")

				-- Show new pins if option is enabled
				if LeaMapsLC["ShowIcons"] == "On" then

					-- Make new pins
					local pMapID = WorldMapFrame.mapID
					if PinData[pMapID] then
						local count = #PinData[pMapID]
						for i = 1, count do

							-- Do nothing if pinInfo has no entry for zone we are looking at
							local pinInfo = PinData[pMapID][i]
							if not pinInfo then return nil end

							-- Get POI if any quest requirements have been met
							if not pinInfo[6] or pinInfo[6] and not pinInfo[7] and IsQuestFlaggedCompleted(pinInfo[6]) or pinInfo[6] and pinInfo[7] and IsQuestFlaggedCompleted(pinInfo[6]) and not IsQuestFlaggedCompleted(pinInfo[7]) then
								if playerFaction == "Alliance" and pinInfo[5] ~= pHTex or playerFaction == "Horde" and pinInfo[5] ~= pATex then
									local myPOI = {}
									myPOI["position"] = CreateVector2D(pinInfo[1] / 100, pinInfo[2] / 100)
									myPOI["name"] = pinInfo[3]
									myPOI["description"] = pinInfo[4]
									myPOI["atlasName"] = pinInfo[5]
									self:GetMap():AcquirePin("LeaMapsGlobalPinTemplate", myPOI)
								end
							end
						end
					end

				end

			end

			LeaMapsGlobalPinMixin = BaseMapPoiPinMixin:CreateSubPin("PIN_FRAME_LEVEL_DUNGEON_ENTRANCE")

			function LeaMapsGlobalPinMixin:OnAcquired(myInfo)
				BaseMapPoiPinMixin.OnAcquired(self, myInfo)
			end

			WorldMapFrame:AddDataProvider(LeaMix)

			-- Toggle icons when option is clicked
			LeaMapsCB["ShowIcons"]:HookScript("OnClick", function() LeaMix:RefreshAllData() end)

		end

		----------------------------------------------------------------------
		-- Reveal unexplored areas
		----------------------------------------------------------------------

		-- Dont reveal garrisons for player faction
		if playerFaction == "Alliance" then
			-- Remove Alliance garrison texture
			Leatrix_Maps["Reveal"][556]["223:279:194:0"] = string.gsub(Leatrix_Maps["Reveal"][556]["223:279:194:0"], "1037663", "")
		elseif playerFaction == "Horde" then
			-- Remove Horde garrison texture
			Leatrix_Maps["Reveal"][542]["267:257:336:327"] = string.gsub(Leatrix_Maps["Reveal"][542]["267:257:336:327"], "1003342", "")
		end

		-- Dont reveal specific areas
		Leatrix_Maps["Reveal"][521] = nil -- Throne of Thunder
		Leatrix_Maps["Reveal"][1176] = nil -- The Dredge (Darkshore)
		Leatrix_Maps["Reveal"][67]["453:340:0:0"] = nil -- Veiled Sea (Darkshore)

		-- Create table to store revealed overlays
		local overlayTextures = {}

		-- Function to refresh overlays (Blizzard_SharedMapDataProviders\MapExplorationDataProvider)
		local function MapExplorationPin_RefreshOverlays(pin, fullUpdate)
			overlayTextures = {}
			local mapID = WorldMapFrame.mapID; if not mapID then return end
			local artID = C_Map.GetMapArtID(mapID); if not artID or not Leatrix_Maps["Reveal"][artID] then return end
			local LeaMapsZone = Leatrix_Maps["Reveal"][artID]

			-- Store already explored tiles in a table so they can be ignored
			local TileExists = {}
			local exploredMapTextures = C_MapExplorationInfo.GetExploredMapTextures(mapID)
			if exploredMapTextures then
				for i, exploredTextureInfo in ipairs(exploredMapTextures) do
					local key = exploredTextureInfo.textureWidth .. ":" .. exploredTextureInfo.textureHeight .. ":" .. exploredTextureInfo.offsetX .. ":" .. exploredTextureInfo.offsetY
					TileExists[key] = true
				end
			end

			-- Get the sizes
			pin.layerIndex = pin:GetMap():GetCanvasContainer():GetCurrentLayerIndex()
			local layers = C_Map.GetMapArtLayers(mapID)
			local layerInfo = layers and layers[pin.layerIndex]
			if not layerInfo then return end
			local TILE_SIZE_WIDTH = layerInfo.tileWidth
			local TILE_SIZE_HEIGHT = layerInfo.tileHeight

			-- Show textures if they are in database and have not been explored
			for key, files in pairs(LeaMapsZone) do
				if not TileExists[key] then
					local width, height, offsetX, offsetY = strsplit(":", key)
					local fileDataIDs = { strsplit(",", files) }
					local numTexturesWide = ceil(width/TILE_SIZE_WIDTH)
					local numTexturesTall = ceil(height/TILE_SIZE_HEIGHT)
					local texturePixelWidth, textureFileWidth, texturePixelHeight, textureFileHeight
					for j = 1, numTexturesTall do
						if ( j < numTexturesTall ) then
							texturePixelHeight = TILE_SIZE_HEIGHT
							textureFileHeight = TILE_SIZE_HEIGHT
						else
							texturePixelHeight = mod(height, TILE_SIZE_HEIGHT)
							if ( texturePixelHeight == 0 ) then
								texturePixelHeight = TILE_SIZE_HEIGHT
							end
							textureFileHeight = 16
							while(textureFileHeight < texturePixelHeight) do
								textureFileHeight = textureFileHeight * 2
							end
						end
						for k = 1, numTexturesWide do
							local texture = pin.overlayTexturePool:Acquire()
							if ( k < numTexturesWide ) then
								texturePixelWidth = TILE_SIZE_WIDTH
								textureFileWidth = TILE_SIZE_WIDTH
							else
								texturePixelWidth = mod(width, TILE_SIZE_WIDTH)
								if ( texturePixelWidth == 0 ) then
									texturePixelWidth = TILE_SIZE_WIDTH
								end
								textureFileWidth = 16
								while(textureFileWidth < texturePixelWidth) do
									textureFileWidth = textureFileWidth * 2
								end
							end
							texture:SetSize(texturePixelWidth, texturePixelHeight)
							texture:SetTexCoord(0, texturePixelWidth/textureFileWidth, 0, texturePixelHeight/textureFileHeight)
							texture:SetPoint("TOPLEFT", offsetX + (TILE_SIZE_WIDTH * (k-1)), -(offsetY + (TILE_SIZE_HEIGHT * (j - 1))))
							texture:SetTexture(tonumber(fileDataIDs[((j - 1) * numTexturesWide) + k]), nil, nil, "TRILINEAR")
							texture:SetDrawLayer("ARTWORK", -1)
							if LeaMapsLC["RevealMap"] == "On" then
								texture:Show()
								if fullUpdate then
									pin.textureLoadGroup:AddTexture(texture)
								end
							else
								texture:Hide()
							end
							if LeaMapsLC["RevTint"] == "On" then
								texture:SetVertexColor(LeaMapsLC["tintRed"], LeaMapsLC["tintGreen"], LeaMapsLC["tintBlue"], LeaMapsLC["tintAlpha"])
							end
							tinsert(overlayTextures, texture)
						end
					end
				end
			end
		end

		-- Reset texture color and alpha
		local function TexturePool_ResetVertexColor(pool, texture)
			texture:SetVertexColor(1, 1, 1)
			texture:SetAlpha(1)
			return TexturePool_HideAndClearAnchors(pool, texture)
		end

		-- Show overlays on startup
		for pin in WorldMapFrame:EnumeratePinsByTemplate("MapExplorationPinTemplate") do
			hooksecurefunc(pin, "RefreshOverlays", MapExplorationPin_RefreshOverlays)
			pin.overlayTexturePool.resetterFunc = TexturePool_ResetVertexColor
		end

		-- Toggle overlays if reveal option is clicked
		LeaMapsCB["RevealMap"]:HookScript("OnClick", function()
			if LeaMapsLC["RevealMap"] == "On" then 
				for i = 1, #overlayTextures  do
					overlayTextures[i]:Show()
				end
			else
				for i = 1, #overlayTextures  do
					overlayTextures[i]:Hide()
				end	
			end
		end)

		-- Create tint frame
		local tintFrame = LeaMapsLC:CreatePanel("Show Unexplored Areas", "tintFrame")

		-- Add controls
		LeaMapsLC:MakeTx(tintFrame, "Settings", 16, -72)
		LeaMapsLC:MakeCB(tintFrame, "RevTint", "Tint unexplored areas", 16, -92, false, "If checked, unexplored areas will be tinted.")
		LeaMapsLC:MakeSL(tintFrame, "tintRed", "Red", "Drag to set the amount of red.", 0, 1, 0.1, 36, -142, "%.1f")
		LeaMapsLC:MakeSL(tintFrame, "tintGreen", "Green", "Drag to set the amount of green.", 0, 1, 0.1, 36, -192, "%.1f")
		LeaMapsLC:MakeSL(tintFrame, "tintBlue", "Blue", "Drag to set the amount of blue.", 0, 1, 0.1, 206, -142, "%.1f")
		LeaMapsLC:MakeSL(tintFrame, "tintAlpha", "Opacity", "Drag to set the opacity.", 0.1, 1, 0.1, 206, -192, "%.1f")

		-- Add preview color block
		local prvTitle = LeaMapsLC:MakeWD(tintFrame, "Preview", 386, -130); prvTitle:Hide()
		tintFrame.preview = tintFrame:CreateTexture(nil, "ARTWORK")
		tintFrame.preview:SetSize(50, 50)
		tintFrame.preview:SetPoint("TOPLEFT", prvTitle, "TOPLEFT", 0, -20)

		-- Function to set tint color
		local function SetTintCol()
			tintFrame.preview:SetColorTexture(LeaMapsLC["tintRed"], LeaMapsLC["tintGreen"], LeaMapsLC["tintBlue"], LeaMapsLC["tintAlpha"])
			-- Set slider values
			LeaMapsCB["tintRed"].f:SetFormattedText("%.0f%%", LeaMapsLC["tintRed"] * 100)
			LeaMapsCB["tintGreen"].f:SetFormattedText("%.0f%%", LeaMapsLC["tintGreen"] * 100)
			LeaMapsCB["tintBlue"].f:SetFormattedText("%.0f%%", LeaMapsLC["tintBlue"] * 100)
			LeaMapsCB["tintAlpha"].f:SetFormattedText("%.0f%%", LeaMapsLC["tintAlpha"] * 100)
			-- Set tint
			if LeaMapsLC["RevTint"] == "On" then
				-- Enable tint
				for i = 1, #overlayTextures  do
					overlayTextures[i]:SetVertexColor(LeaMapsLC["tintRed"], LeaMapsLC["tintGreen"], LeaMapsLC["tintBlue"], LeaMapsLC["tintAlpha"])
				end
				-- Enable controls
				LeaMapsCB["tintRed"]:Enable(); LeaMapsCB["tintRed"]:SetAlpha(1.0)
				LeaMapsCB["tintGreen"]:Enable(); LeaMapsCB["tintGreen"]:SetAlpha(1.0)
				LeaMapsCB["tintBlue"]:Enable(); LeaMapsCB["tintBlue"]:SetAlpha(1.0)
				LeaMapsCB["tintAlpha"]:Enable(); LeaMapsCB["tintAlpha"]:SetAlpha(1.0)
				prvTitle:SetAlpha(1.0); tintFrame.preview:SetAlpha(1.0)
			else
				-- Disable tint
				for i = 1, #overlayTextures  do
					overlayTextures[i]:SetVertexColor(1, 1, 1)
					overlayTextures[i]:SetAlpha(1.0)
				end
				-- Disable controls
				LeaMapsCB["tintRed"]:Disable(); LeaMapsCB["tintRed"]:SetAlpha(0.3)
				LeaMapsCB["tintGreen"]:Disable(); LeaMapsCB["tintGreen"]:SetAlpha(0.3)
				LeaMapsCB["tintBlue"]:Disable(); LeaMapsCB["tintBlue"]:SetAlpha(0.3)
				LeaMapsCB["tintAlpha"]:Disable(); LeaMapsCB["tintAlpha"]:SetAlpha(0.3)
				prvTitle:SetAlpha(0.3); tintFrame.preview:SetAlpha(0.3)
			end
		end

		-- Set tint properties when controls are changed and on startup
		LeaMapsCB["RevTint"]:HookScript("OnClick", SetTintCol)
		LeaMapsCB["tintRed"]:HookScript("OnMouseWheel", SetTintCol)
		LeaMapsCB["tintRed"]:HookScript("OnValueChanged", SetTintCol)
		LeaMapsCB["tintGreen"]:HookScript("OnMouseWheel", SetTintCol)
		LeaMapsCB["tintGreen"]:HookScript("OnValueChanged", SetTintCol)
		LeaMapsCB["tintBlue"]:HookScript("OnMouseWheel", SetTintCol)
		LeaMapsCB["tintBlue"]:HookScript("OnValueChanged", SetTintCol)
		LeaMapsCB["tintAlpha"]:HookScript("OnMouseWheel", SetTintCol)
		LeaMapsCB["tintAlpha"]:HookScript("OnValueChanged", SetTintCol)
		SetTintCol()

		-- Back to Main Menu button click
		tintFrame.b:HookScript("OnClick", function()
			tintFrame:Hide()
			LeaMapsLC["PageF"]:Show()
		end)

		-- Reset button click
		tintFrame.r:HookScript("OnClick", function()
			LeaMapsLC["RevTint"] = "On"
			LeaMapsLC["tintRed"] = 0.6
			LeaMapsLC["tintGreen"] = 0.6
			LeaMapsLC["tintBlue"] = 1
			LeaMapsLC["tintAlpha"] = 1
			SetTintCol()
			tintFrame:Hide(); tintFrame:Show()
		end)

		-- Show tint configuration panel when configuration button is clicked
		LeaMapsCB["RevTintBtn"]:HookScript("OnClick", function()
			if IsShiftKeyDown() and IsControlKeyDown() then
				-- Preset profile
				LeaMapsLC["RevTint"] = "On"
				LeaMapsLC["tintRed"] = 0.6
				LeaMapsLC["tintGreen"] = 0.6
				LeaMapsLC["tintBlue"] = 1
				LeaMapsLC["tintAlpha"] = 1
				SetTintCol()
				if tintFrame:IsShown() then tintFrame:Hide(); tintFrame:Show(); end
			else
				tintFrame:Show()
				LeaMapsLC["PageF"]:Hide()
			end
		end)

		----------------------------------------------------------------------
		-- Toggle Leatrix Maps panel with help button
		----------------------------------------------------------------------

		WorldMapFrame.BorderFrame.Tutorial:SetScript("OnEnter", function() end)
		WorldMapFrame.BorderFrame.Tutorial:SetScript("OnClick", function() 
			-- Prevent options panel from showing if a game options panel is showing
			if InterfaceOptionsFrame:IsShown() or VideoOptionsFrame:IsShown() or ChatConfigFrame:IsShown() then return end
			-- Prevent options panel from showing if Blizzard Store is showing
			if StoreFrame and StoreFrame:GetAttribute("isshown") then return end
			-- Toggle the options panel if game options panel is not showing
			if LeaMapsLC:IsMapsShowing() then
				LeaMapsLC["PageF"]:Hide()
				LeaMapsLC:HideConfigPanels()
			else
				LeaMapsLC["PageF"]:Show()
			end
		end)
		SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_WORLD_MAP_FRAME, true)

		----------------------------------------------------------------------
		-- Show minimap icon
		----------------------------------------------------------------------

		do

			-- Minimap button click function
			local function MiniBtnClickFunc(arg1)
				-- Prevent options panel from showing if Blizzard options panel is showing
				if InterfaceOptionsFrame:IsShown() or VideoOptionsFrame:IsShown() or ChatConfigFrame:IsShown() then return end
				-- No modifier key toggles the options panel
				if LeaMapsLC:IsMapsShowing() then
					LeaMapsLC["PageF"]:Hide()
					LeaMapsLC:HideConfigPanels()
				else
					LeaMapsLC["PageF"]:Show()
				end

			end

			-- Create minimap button using LibDBIcon
			local ldb = LibStub("LibDataBroker-1.1", true)
			local miniButton = ldb:NewDataObject("Leatrix_Maps", {
				type = "launcher",
				icon = "Interface\\Worldmap\\UI-World-Icon",
				OnClick = function(self, btn)
					MiniBtnClickFunc(btn)
				end,
				OnTooltipShow = function(tooltip)
					if not tooltip or not tooltip.AddLine then return end
					tooltip:AddLine("Leatrix Maps")
				end,
			})
			local icon = LibStub("LibDBIcon-1.0", true)
			icon:Register("Leatrix_Maps", miniButton, LeaMapsDB)

			-- Function to toggle LibDBIcon
			local function SetLibDBIconFunc()
				if LeaMapsLC["ShowMinimapIcon"] == "On" then
					icon:Show("Leatrix_Maps")
				else
					icon:Hide("Leatrix_Maps")
				end
			end

			-- Set LibDBIcon when option is clicked and on startup
			LeaMapsCB["ShowMinimapIcon"]:HookScript("OnClick", SetLibDBIconFunc)
			SetLibDBIconFunc()

		end

		----------------------------------------------------------------------
		-- Show memory usage
		----------------------------------------------------------------------

		do

			-- Show memory usage stat
			local function ShowMemoryUsage(frame, anchor, x, y)

				-- Create frame
				local memframe = CreateFrame("FRAME", nil, frame)
				memframe:ClearAllPoints()
				memframe:SetPoint(anchor, x, y)
				memframe:SetWidth(100)
				memframe:SetHeight(20)

				-- Create labels
				local pretext = memframe:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
				pretext:SetPoint("TOPLEFT", 0, 0)
				pretext:SetText(L["Memory Usage"])

				local memtext = memframe:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
				memtext:SetPoint("TOPLEFT", 0, 0 - 30)

				-- Create stat
				local memstat = memframe:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
				memstat:SetPoint("BOTTOMLEFT", memtext, "BOTTOMRIGHT")
				memstat:SetText("(calculating...)")

				-- Create update script
				local memtime = -1
				memframe:SetScript("OnUpdate", function(self, elapsed)
					if memtime > 2 or memtime == -1 then
						UpdateAddOnMemoryUsage()
						memtext = GetAddOnMemoryUsage("Leatrix_Maps")
						memtext = math.floor(memtext + .5) .. " KB"
						memstat:SetText(memtext)
						memtime = 0
					end
					memtime = memtime + elapsed
				end)

			end

			-- ShowMemoryUsage(LeaMapsLC["PageF"], "TOPLEFT", 16, -282)

		end

		----------------------------------------------------------------------
		-- Create panel in game options panel
		----------------------------------------------------------------------

		do

			local interPanel = CreateFrame("FRAME")
			interPanel.name = "Leatrix Maps"

			local maintitle = LeaMapsLC:MakeTx(interPanel, "Leatrix Maps", 0, 0)
			maintitle:SetFont(maintitle:GetFont(), 72)
			maintitle:ClearAllPoints()
			maintitle:SetPoint("TOP", 0, -72)

			local subTitle = LeaMapsLC:MakeTx(interPanel, "curseforge.com/wow/addons/leatrix-maps", 0, 0)
			subTitle:SetFont(subTitle:GetFont(), 20)
			subTitle:ClearAllPoints()
			subTitle:SetPoint("BOTTOM", 0, 72)

			local slashTitle = LeaMapsLC:MakeTx(interPanel, "/ltm", 0, 0)
			slashTitle:SetFont(slashTitle:GetFont(), 72)
			slashTitle:ClearAllPoints()
			slashTitle:SetPoint("BOTTOM", subTitle, "TOP", 0, 40)

			local pTex = interPanel:CreateTexture(nil, "BACKGROUND")
			pTex:SetAllPoints()
			pTex:SetTexture("Interface\\GLUES\\Models\\UI_MainMenu\\swordgradient2")
			pTex:SetAlpha(0.2)
			pTex:SetTexCoord(0, 1, 1, 0)

			InterfaceOptions_AddCategory(interPanel)

		end

		----------------------------------------------------------------------
		-- Final code
		----------------------------------------------------------------------

		-- Show first run message
		if not LeaMapsDB["FirstRunMessageSeen"] then
			C_Timer.After(1, function()
				LeaMapsLC:Print(L["Enter"] .. " |cff00ff00" .. "/ltm" .. "|r " .. L["or click the minimap button to open Leatrix Maps."])
				LeaMapsDB["FirstRunMessageSeen"] = true
			end)
		end

		-- Release memory
		LeaMapsLC.MainFunc = nil

	end

	----------------------------------------------------------------------
	-- L10: Functions
	----------------------------------------------------------------------

	-- Function to add textures to panels
	function LeaMapsLC:CreateBar(name, parent, width, height, anchor, r, g, b, alp, tex)
		local ft = parent:CreateTexture(nil, "BORDER")
		ft:SetTexture(tex)
		ft:SetSize(width, height)  
		ft:SetPoint(anchor)
		ft:SetVertexColor(r ,g, b, alp)
		if name == "MainTexture" then
			ft:SetTexCoord(0.09, 1, 0, 1)
		end
	end

	-- Create a configuration panel
	function LeaMapsLC:CreatePanel(title, globref)

		-- Create the panel
		local Side = CreateFrame("Frame", nil, UIParent)

		-- Make it a system frame
		--_G["LeaMapsGlobalPanel_" .. globref] = Side
		--table.insert(UISpecialFrames, "LeaMapsGlobalPanel_" .. globref)

		-- Store it in the configuration panel table
		tinsert(LeaConfigList, Side)

		-- Set frame parameters
		Side:Hide()
		Side:SetSize(470, 340)
		Side:SetClampedToScreen(true)
		Side:SetFrameStrata("FULLSCREEN_DIALOG")
		Side:SetFrameLevel(20)

		-- Set the background color
		Side.t = Side:CreateTexture(nil, "BACKGROUND")
		Side.t:SetAllPoints()
		Side.t:SetColorTexture(0.05, 0.05, 0.05, 0.9)

		-- Add a close Button
		Side.c = CreateFrame("Button", nil, Side, "UIPanelCloseButton") 
		Side.c:SetSize(30, 30)
		Side.c:SetPoint("TOPRIGHT", 0, 0)
		Side.c:SetScript("OnClick", function() Side:Hide() end)

		-- Add reset, help and back buttons
		Side.r = LeaMapsLC:CreateButton("ResetButton", Side, "Reset", "BOTTOMLEFT", 16, 60, 25, "Click to reset the settings on this page.")
		Side.b = LeaMapsLC:CreateButton("BackButton", Side, "Back to Main Menu", "BOTTOMRIGHT", -16, 60, 25, "Click to return to the main menu.")

		-- Add a reload button and synchronise it with the main panel reload button
		local reloadb = LeaMapsLC:CreateButton("ConfigReload", Side, "Reload", "BOTTOMRIGHT", -16, 10, 25)
		LeaMapsLC:LockItem(reloadb, true)
		reloadb:SetScript("OnClick", ReloadUI)

		reloadb.f = reloadb:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
		reloadb.f:SetHeight(32)
		reloadb.f:SetPoint('RIGHT', reloadb, 'LEFT', -10, 0)
		reloadb.f:SetText(LeaMapsCB["ReloadUIButton"].f:GetText())
		reloadb.f:Hide()

		LeaMapsCB["ReloadUIButton"]:HookScript("OnEnable", function()
			LeaMapsLC:LockItem(reloadb, false)
			reloadb.f:Show()
		end)

		LeaMapsCB["ReloadUIButton"]:HookScript("OnDisable", function()
			LeaMapsLC:LockItem(reloadb, true)
			reloadb.f:Hide()
		end)

		-- Set textures
		LeaMapsLC:CreateBar("FootTexture", Side, 470, 48, "BOTTOM", 0.5, 0.5, 0.5, 1.0, "Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
		LeaMapsLC:CreateBar("MainTexture", Side, 470, 293, "TOPRIGHT", 0.7, 0.7, 0.7, 0.7,  "Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")

		-- Allow movement
		Side:EnableMouse(true)
		Side:SetMovable(true)
		Side:RegisterForDrag("LeftButton")
		Side:SetScript("OnDragStart", Side.StartMoving)
		Side:SetScript("OnDragStop", function ()
			Side:StopMovingOrSizing()
			Side:SetUserPlaced(false)
			-- Save panel position
			LeaMapsLC["MainPanelA"], void, LeaMapsLC["MainPanelR"], LeaMapsLC["MainPanelX"], LeaMapsLC["MainPanelY"] = Side:GetPoint()
		end)

		-- Set panel attributes when shown
		Side:SetScript("OnShow", function()
			Side:ClearAllPoints()
			Side:SetPoint(LeaMapsLC["MainPanelA"], UIParent, LeaMapsLC["MainPanelR"], LeaMapsLC["MainPanelX"], LeaMapsLC["MainPanelY"])
		end)

		-- Add title
		Side.f = Side:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
		Side.f:SetPoint('TOPLEFT', 16, -16)
		Side.f:SetText(L[title])

		-- Add description
		Side.v = Side:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
		Side.v:SetHeight(32)
		Side.v:SetPoint('TOPLEFT', Side.f, 'BOTTOMLEFT', 0, -8)
		Side.v:SetPoint('RIGHT', Side, -32, 0)
		Side.v:SetJustifyH('LEFT'); Side.v:SetJustifyV('TOP')
		Side.v:SetText(L["Configuration Panel"])
	
		-- Prevent options panel from showing while side panel is showing
		LeaMapsLC["PageF"]:HookScript("OnShow", function()
			if Side:IsShown() then LeaMapsLC["PageF"]:Hide(); end
		end)

		-- Return the frame
		return Side

	end

	-- Hide configuration panels
	function LeaMapsLC:HideConfigPanels()
		for k, v in pairs(LeaConfigList) do
			v:Hide()
		end
	end

	-- Find out if Leatrix Maps is showing (main panel or config panel)
	function LeaMapsLC:IsMapsShowing()
		if LeaMapsLC["PageF"]:IsShown() then return true end
		for k, v in pairs(LeaConfigList) do
			if v:IsShown() then
				return true
			end
		end
	end

	-- Load a string variable or set it to default if it's not set to "On" or "Off"
	function LeaMapsLC:LoadVarChk(var, def)
		if LeaMapsDB[var] and type(LeaMapsDB[var]) == "string" and LeaMapsDB[var] == "On" or LeaMapsDB[var] == "Off" then
			LeaMapsLC[var] = LeaMapsDB[var]
		else
			LeaMapsLC[var] = def
			LeaMapsDB[var] = def
		end
	end

	-- Load a numeric variable and set it to default if it's not within a given range
	function LeaMapsLC:LoadVarNum(var, def, valmin, valmax)
		if LeaMapsDB[var] and type(LeaMapsDB[var]) == "number" and LeaMapsDB[var] >= valmin and LeaMapsDB[var] <= valmax then
			LeaMapsLC[var] = LeaMapsDB[var]
		else
			LeaMapsLC[var] = def
			LeaMapsDB[var] = def
		end
	end

	-- Load an anchor point variable and set it to default if the anchor point is invalid
	function LeaMapsLC:LoadVarAnc(var, def)
		if LeaMapsDB[var] and type(LeaMapsDB[var]) == "string" and LeaMapsDB[var] == "CENTER" or LeaMapsDB[var] == "TOP" or LeaMapsDB[var] == "BOTTOM" or LeaMapsDB[var] == "LEFT" or LeaMapsDB[var] == "RIGHT" or LeaMapsDB[var] == "TOPLEFT" or LeaMapsDB[var] == "TOPRIGHT" or LeaMapsDB[var] == "BOTTOMLEFT" or LeaMapsDB[var] == "BOTTOMRIGHT" then
			LeaMapsLC[var] = LeaMapsDB[var]
		else
			LeaMapsLC[var] = def
			LeaMapsDB[var] = def
		end
	end

	-- Show tooltips for checkboxes
	function LeaMapsLC:TipSee()
		if not self:IsEnabled() then return end
		GameTooltip:SetOwner(self, "ANCHOR_NONE")
		local parent = self:GetParent()
		local pscale = parent:GetEffectiveScale()
		local gscale = UIParent:GetEffectiveScale()
		local tscale = GameTooltip:GetEffectiveScale()
		local gap = ((UIParent:GetRight() * gscale) - (parent:GetRight() * pscale))
		if gap < (250 * tscale) then
			GameTooltip:SetPoint("TOPRIGHT", parent, "TOPLEFT", 0, 0)
		else
			GameTooltip:SetPoint("TOPLEFT", parent, "TOPRIGHT", 0, 0)
		end
		GameTooltip:SetText(self.tiptext, nil, nil, nil, nil, true)
	end

	-- Show tooltips for configuration buttons and dropdown menus
	function LeaMapsLC:ShowTooltip()
		if not self:IsEnabled() then return end
		GameTooltip:SetOwner(self, "ANCHOR_NONE")
		local parent = LeaMapsLC["PageF"]
		local pscale = parent:GetEffectiveScale()
		local gscale = UIParent:GetEffectiveScale()
		local tscale = GameTooltip:GetEffectiveScale()
		local gap = ((UIParent:GetRight() * gscale) - (LeaMapsLC["PageF"]:GetRight() * pscale))
		if gap < (250 * tscale) then
			GameTooltip:SetPoint("TOPRIGHT", parent, "TOPLEFT", 0, 0)
		else
			GameTooltip:SetPoint("TOPLEFT", parent, "TOPRIGHT", 0, 0)
		end
		GameTooltip:SetText(self.tiptext, nil, nil, nil, nil, true)
	end

	-- Print text
	function LeaMapsLC:Print(text)
		DEFAULT_CHAT_FRAME:AddMessage(L[text], 1.0, 0.85, 0.0)
	end

	-- Lock and unlock an item
	function LeaMapsLC:LockItem(item, lock)
		if lock then
			item:Disable()
			item:SetAlpha(0.3)
		else
			item:Enable()
			item:SetAlpha(1.0)
		end
	end

	-- Function to set lock state for configuration buttons
	function LeaMapsLC:LockOption(option, item, reloadreq)
		if reloadreq then
			-- Option requires UI reload
			if LeaMapsLC[option] ~= LeaMapsDB[option] or LeaMapsLC[option] == "Off" then
				LeaMapsLC:LockItem(LeaMapsCB[item], true)
			else
				LeaMapsLC:LockItem(LeaMapsCB[item], false)
			end

		else
			-- Option does not require UI reload
			if LeaMapsLC[option] == "Off" then
				LeaMapsLC:LockItem(LeaMapsCB[item], true)
			else
				LeaMapsLC:LockItem(LeaMapsCB[item], false)
			end
		end
	end

	-- Set lock state for configuration buttons
	function LeaMapsLC:SetDim()
		LeaMapsLC:LockOption("RevealMap", "RevTintBtn", false)		-- Reveal map
		LeaMapsLC:LockOption("UnlockMap", "UnlockMapBtn", true)		-- Unlock map frame
		LeaMapsLC:LockOption("ShowCoords", "ShowCoordsBtn", false)	-- Show coordinates
	end

	-- Create a standard button
	function LeaMapsLC:CreateButton(name, frame, label, anchor, x, y, height, tip)
		local mbtn = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
		LeaMapsCB[name] = mbtn
		mbtn:SetHeight(height)
		mbtn:SetPoint(anchor, x, y)
		mbtn:SetHitRectInsets(0, 0, 0, 0)
		mbtn:SetText(L[label])

		-- Create fontstring and set button width based on it
		mbtn.f = mbtn:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
		mbtn.f:SetText(L[label])
		mbtn:SetWidth(mbtn.f:GetStringWidth() + 20)

		-- Tooltip handler
		mbtn.tiptext = L[tip]
		mbtn:SetScript("OnEnter", LeaMapsLC.TipSee)
		mbtn:SetScript("OnLeave", GameTooltip_Hide)

		-- Set skinned button textures
		mbtn:SetNormalTexture("Interface\\AddOns\\Leatrix_Maps\\Leatrix_Maps.blp")
		mbtn:GetNormalTexture():SetTexCoord(0.5, 1, 0, 1)
		mbtn:SetHighlightTexture("Interface\\AddOns\\Leatrix_Maps\\Leatrix_Maps.blp")
		mbtn:GetHighlightTexture():SetTexCoord(0, 0.5, 0, 1)

		-- Hide the default textures
		mbtn:HookScript("OnShow", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnEnable", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnDisable", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnMouseDown", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnMouseUp", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)

		return mbtn
	end

	-- Set reload button status
	function LeaMapsLC:ReloadCheck()
		if	(LeaMapsLC["UnlockMap"] ~= LeaMapsDB["UnlockMap"])	-- Unlock map
		then
			-- Enable the reload button
			LeaMapsLC:LockItem(LeaMapsCB["ReloadUIButton"], false)
			LeaMapsCB["ReloadUIButton"].f:Show()
		else
			-- Disable the reload button
			LeaMapsLC:LockItem(LeaMapsCB["ReloadUIButton"], true)
			LeaMapsCB["ReloadUIButton"].f:Hide()
		end
	end

	-- Create a subheading
	function LeaMapsLC:MakeTx(frame, title, x, y)
		local text = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
		text:SetPoint("TOPLEFT", x, y)
		text:SetText(L[title])
		return text
	end

	-- Create text
	function LeaMapsLC:MakeWD(frame, title, x, y, width)
		local text = frame:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
		text:SetPoint("TOPLEFT", x, y)
		text:SetJustifyH("LEFT")
		text:SetText(L[title])
		if width then text:SetWidth(width) end
		return text
	end

	-- Create a checkbox control
	function LeaMapsLC:MakeCB(parent, field, caption, x, y, reload, tip)

		-- Create the checkbox
		local Cbox = CreateFrame('CheckButton', nil, parent, "ChatConfigCheckButtonTemplate")
		LeaMapsCB[field] = Cbox
		Cbox:SetPoint("TOPLEFT",x, y)
		Cbox:SetScript("OnEnter", LeaMapsLC.TipSee)
		Cbox:SetScript("OnLeave", GameTooltip_Hide)

		-- Add label and tooltip
		Cbox.f = Cbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
		Cbox.f:SetPoint('LEFT', 24, 0)
		if reload then
			-- Checkbox requires UI reload
			Cbox.f:SetText(L[caption] .. "*")
			Cbox.tiptext = L[tip] .. "|n|n* " .. L["Requires UI reload."]
		else
			-- Checkbox does not require UI reload
			Cbox.f:SetText(L[caption])
			Cbox.tiptext = L[tip]
		end

		-- Set label parameters
		Cbox.f:SetJustifyH("LEFT")
		Cbox.f:SetWordWrap(false)

		-- Set maximum label width
		if parent == LeaMapsLC["PageF"] then
			-- Main panel checkbox labels
			if Cbox.f:GetWidth() > 172 then
				Cbox.f:SetWidth(172)
			end
			-- Set checkbox click width
			if Cbox.f:GetStringWidth() > 172 then
				Cbox:SetHitRectInsets(0, -162, 0, 0)
			else
				Cbox:SetHitRectInsets(0, -Cbox.f:GetStringWidth() + 4, 0, 0)
			end
		else
			-- Configuration panel checkbox labels (other checkboxes either have custom functions or blank labels)
			if Cbox.f:GetWidth() > 322 then
				Cbox.f:SetWidth(322)
			end
			-- Set checkbox click width
			if Cbox.f:GetStringWidth() > 322 then
				Cbox:SetHitRectInsets(0, -312, 0, 0)
			else
				Cbox:SetHitRectInsets(0, -Cbox.f:GetStringWidth() + 4, 0, 0)
			end
		end

		-- Set default checkbox state and click area
		Cbox:SetScript('OnShow', function(self)
			if LeaMapsLC[field] == "On" then
				self:SetChecked(true)
			else
				self:SetChecked(false)
			end
		end)

		-- Process clicks
		Cbox:SetScript('OnClick', function()
			if Cbox:GetChecked() then
				LeaMapsLC[field] = "On"
			else
				LeaMapsLC[field] = "Off"
			end
			LeaMapsLC:SetDim() -- Lock invalid options
			LeaMapsLC:ReloadCheck()
		end)
	end

	-- Create configuration button
	function LeaMapsLC:CfgBtn(name, parent)
		local CfgBtn = CreateFrame("BUTTON", nil, parent)
		LeaMapsCB[name] = CfgBtn
		CfgBtn:SetWidth(20)
		CfgBtn:SetHeight(20)
		CfgBtn:SetPoint("LEFT", parent.f, "RIGHT", 0, 0)

		CfgBtn.t = CfgBtn:CreateTexture(nil, "BORDER")
		CfgBtn.t:SetAllPoints()
		CfgBtn.t:SetTexture("Interface\\WorldMap\\Gear_64.png")
		CfgBtn.t:SetTexCoord(0, 0.50, 0, 0.50)
		CfgBtn.t:SetVertexColor(1.0, 0.82, 0, 1.0)

		CfgBtn:SetHighlightTexture("Interface\\WorldMap\\Gear_64.png")
		CfgBtn:GetHighlightTexture():SetTexCoord(0, 0.50, 0, 0.50)

		CfgBtn.tiptext = L["Click to configure the settings for this option."]
		CfgBtn:SetScript("OnEnter", LeaMapsLC.ShowTooltip)
		CfgBtn:SetScript("OnLeave", GameTooltip_Hide)
	end

	-- Create a slider control
	function LeaMapsLC:MakeSL(frame, field, label, caption, low, high, step, x, y, form)

		-- Create slider control
		local Slider = CreateFrame("Slider", "LeaMapsGlobalSlider" .. field, frame, "OptionssliderTemplate")
		LeaMapsCB[field] = Slider
		Slider:SetMinMaxValues(low, high)
		Slider:SetValueStep(step)
		Slider:EnableMouseWheel(true)
		Slider:SetPoint('TOPLEFT', x,y)
		Slider:SetWidth(100)
		Slider:SetHeight(20)
		Slider:SetHitRectInsets(0, 0, 0, 0)
		Slider.tiptext = L[caption]
		Slider:SetScript("OnEnter", LeaMapsLC.TipSee)
		Slider:SetScript("OnLeave", GameTooltip_Hide)

		-- Remove slider text
		_G[Slider:GetName().."Low"]:SetText('')
		_G[Slider:GetName().."High"]:SetText('')

		-- Set label
		_G[Slider:GetName().."Text"]:SetText(L[label])

		-- Create slider label
		Slider.f = Slider:CreateFontString(nil, 'BACKGROUND')
		Slider.f:SetFontObject('GameFontHighlight')
		Slider.f:SetPoint('LEFT', Slider, 'RIGHT', 12, 0)
		Slider.f:SetFormattedText("%.2f", Slider:GetValue())

		-- Process mousewheel scrolling
		Slider:SetScript("OnMouseWheel", function(self, arg1)
			if Slider:IsEnabled() then
				local step = step * arg1
				local value = self:GetValue()
				if step > 0 then
					self:SetValue(min(value + step, high))
				else
					self:SetValue(max(value + step, low))
				end
			end
		end)

		-- Process value changed
		Slider:SetScript("OnValueChanged", function(self, value)
			local value = floor((value - low) / step + 0.5) * step + low
			Slider.f:SetFormattedText(form, value)
			LeaMapsLC[field] = value
		end)

		-- Set slider value when shown
		Slider:SetScript("OnShow", function(self)
			self:SetValue(LeaMapsLC[field])
		end)

	end

	----------------------------------------------------------------------
	-- Stop error frame
	----------------------------------------------------------------------

	-- Create stop error frame
	local stopFrame = CreateFrame("FRAME", nil, UIParent)
	stopFrame:ClearAllPoints()
	stopFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	stopFrame:SetSize(370, 150)
	stopFrame:SetFrameStrata("FULLSCREEN_DIALOG")
	stopFrame:SetFrameLevel(500)
	stopFrame:SetClampedToScreen(true)
	stopFrame:EnableMouse(true)
	stopFrame:SetMovable(true)
	stopFrame:Hide()
	stopFrame:RegisterForDrag("LeftButton")
	stopFrame:SetScript("OnDragStart", stopFrame.StartMoving)
	stopFrame:SetScript("OnDragStop", function()
		stopFrame:StopMovingOrSizing()
		stopFrame:SetUserPlaced(false)
	end)

	-- Add background color
	stopFrame.t = stopFrame:CreateTexture(nil, "BACKGROUND")
	stopFrame.t:SetAllPoints()
	stopFrame.t:SetColorTexture(0.05, 0.05, 0.05, 0.9)

	-- Add textures
	stopFrame.mt = stopFrame:CreateTexture(nil, "BORDER")
	stopFrame.mt:SetTexture("Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
	stopFrame.mt:SetSize(370, 103)
	stopFrame.mt:SetPoint("TOPRIGHT")
	stopFrame.mt:SetVertexColor(0.5, 0.5, 0.5, 1.0)

	stopFrame.ft = stopFrame:CreateTexture(nil, "BORDER")
	stopFrame.ft:SetTexture("Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
	stopFrame.ft:SetSize(370, 48)
	stopFrame.ft:SetPoint("BOTTOM")
	stopFrame.ft:SetVertexColor(0.5, 0.5, 0.5, 1.0)

	LeaMapsLC:MakeTx(stopFrame, "Leatrix Maps", 16, -12)
	LeaMapsLC:MakeWD(stopFrame, "A stop error has occurred but no need to worry.  It can happen from time to time.  Click the reload button to resolve it.", 16, -32, 338)

	-- Add reload UI Button
	local stopRelBtn = LeaMapsLC:CreateButton("StopReloadButton", stopFrame, "Reload", "BOTTOMRIGHT", -16, 10, 25, "")
	stopRelBtn:SetScript("OnClick", ReloadUI)
	stopRelBtn.f = stopRelBtn:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	stopRelBtn.f:SetHeight(32)
	stopRelBtn.f:SetPoint('RIGHT', stopRelBtn, 'LEFT', -10, 0)
	stopRelBtn.f:SetText(L["Your UI needs to be reloaded."])
	stopRelBtn:Hide(); stopRelBtn:Show()

	-- Add close Button
	local stopFrameClose = CreateFrame("Button", nil, stopFrame, "UIPanelCloseButton") 
	stopFrameClose:SetSize(30, 30)
	stopFrameClose:SetPoint("TOPRIGHT", 0, 0)

	----------------------------------------------------------------------
	-- L20: Commands
	----------------------------------------------------------------------

	-- Slash command function
	local function SlashFunc(str)
		local str, arg1, arg2, arg3 = strsplit(" ", string.lower(str:gsub("%s+", " ")))
		if str and str ~= "" then
			-- Traverse parameters
			if str == "reset" then
				-- Reset the configuration panel position
				LeaMapsLC["MainPanelA"], LeaMapsLC["MainPanelR"], LeaMapsLC["MainPanelX"], LeaMapsLC["MainPanelY"] = "CENTER", "CENTER", 0, 0
				if LeaMapsLC["PageF"]:IsShown() then LeaMapsLC["PageF"]:Hide() LeaMapsLC["PageF"]:Show() end
				return
			elseif str == "wipe" then
				-- Wipe all settings
				SetCVar("mapFade", "1")
				wipe(LeaMapsDB)
				LeaMapsLC["NoSaveSettings"] = true
				ReloadUI()
			elseif str == "debug" then
				-- Toggle debug mode
				if LeaMapsLC["DebugMode"] then
					LeaMapsLC["DebugMode"] = nil
					if GetCVar("showDungeonEntrancesOnMap") ~= "0" then
						SetCVar("showDungeonEntrancesOnMap", "0")
					end
					LeaMapsLC:Print(L["Debug"] .. "|cffffffff " .. L["disabled"] .. "|r.")
				else
					LeaMapsLC["DebugMode"] = true
					LeaMapsLC:Print(L["Debug"] .. "|cffffffff " .. L["enabled"] .. "|r.")
				end
				return
			elseif str == "setmap" then
				-- Set map to map ID
				arg1 = tonumber(arg1)
				if arg1 and arg1 > 0 and arg1 < 99999 and C_Map.GetMapArtLayers(arg1) then
					WorldMapFrame:SetMapID(arg1)
				else
					LeaMapsLC:Print("Invalid map ID.")
				end
				return
			elseif str == "hadmin" then
				-- Show admin commands
				LeaMapsLC:Print("reset - Reset panel position")
				LeaMapsLC:Print("wipe - Wipe addon settings")
				LeaMapsLC:Print("debug - Lets you enable dungeon icons in world map settings")
				LeaMapsLC:Print("setmap <id> - Show map ID <id>")
				LeaMapsLC:Print("admin - Load admin profile")
				return
			elseif str == "admin" then
				-- Preset profile (reload required)
				LeaMapsLC["NoSaveSettings"] = true
				wipe(LeaMapsDB)

				-- Mechanics
				LeaMapsDB["UnlockMap"] = "Off"
				LeaMapsDB["EnableMovement"] = "On"
				LeaMapsDB["MapScale"] = 1.0
				LeaMapsDB["MaxMapScale"] = 0.9
				LeaMapsDB["StickyMapFrame"] = "Off"
				LeaMapsDB["RememberZoom"] = "On"
				LeaMapsDB["NoMapFade"] = "On"
				LeaMapsDB["NoMapEmote"] = "On"
				LeaMapsDB["MapPosA"] = "TOPLEFT"
				LeaMapsDB["MapPosR"] = "TOPLEFT"
				LeaMapsDB["MapPosX"] = 16
				LeaMapsDB["MapPosY"] = -94
				LeaMapsDB["MaxMapPosA"] = "CENTER"
				LeaMapsDB["MaxMapPosR"] = "CENTER"
				LeaMapsDB["MaxMapPosX"] = 0
				LeaMapsDB["MaxMapPosY"] = 0

				-- Elements
				LeaMapsDB["RevealMap"] = "On"
				LeaMapsDB["RevTint"] = "On"
				LeaMapsDB["tintRed"] = 0.6
				LeaMapsDB["tintGreen"] = 0.6
				LeaMapsDB["tintBlue"] = 1.0
				LeaMapsDB["tintAlpha"] = 1.0
				LeaMapsDB["ShowIcons"] = "On"
				LeaMapsDB["ShowCoords"] = "On"
				LeaMapsDB["CoordsLargeFont"] = "Off"
				LeaMapsDB["CoordsBackground"] = "On"

				-- Settings
				LeaMapsDB["ShowMinimapIcon"] = "On"
				LeaMapsDB["minimapPos"] = 204 -- LeaMapsDB

				ReloadUI()
			elseif str == "help" then
				-- Show available commands
				LeaMapsLC:Print("Leatrix Maps" .. "|n")
				LeaMapsLC:Print(L["Version"] .. " " .. LeaMapsLC["AddonVer"] .. "|n|n")
				LeaMapsLC:Print("/ltm reset - Reset the panel position.")
				LeaMapsLC:Print("/ltm wipe - Wipe all settings and reload.")
				LeaMapsLC:Print("/ltm help - Show this information.")
				return
			else
				-- Invalid command entered
				LeaMapsLC:Print("Invalid command.  Enter /ltm help for help.")
				return
			end
		else
			-- Prevent options panel from showing if a game options panel is showing
			if InterfaceOptionsFrame:IsShown() or VideoOptionsFrame:IsShown() or ChatConfigFrame:IsShown() then return end
			-- Prevent options panel from showing if Blizzard Store is showing
			if StoreFrame and StoreFrame:GetAttribute("isshown") then return end
			-- Toggle the options panel if game options panel is not showing
			if LeaMapsLC:IsMapsShowing() then
				LeaMapsLC["PageF"]:Hide()
				LeaMapsLC:HideConfigPanels()
			else
				LeaMapsLC["PageF"]:Show()
			end
		end
	end

	-- Add slash commands
	_G.SLASH_Leatrix_Maps1 = "/ltm"
	_G.SLASH_Leatrix_Maps2 = "/leamaps" 
	SlashCmdList["Leatrix_Maps"] = function(self)
		-- Run slash command function
		SlashFunc(self)
		-- Redirect tainted variables
		RunScript('ACTIVE_CHAT_EDIT_BOX = ACTIVE_CHAT_EDIT_BOX')
		RunScript('LAST_ACTIVE_CHAT_EDIT_BOX = LAST_ACTIVE_CHAT_EDIT_BOX')
	end

	----------------------------------------------------------------------
	-- L30: Events
	----------------------------------------------------------------------

	-- Create event frame
	local eFrame = CreateFrame("FRAME")
	eFrame:RegisterEvent("ADDON_LOADED")
	eFrame:RegisterEvent("PLAYER_LOGIN")
	eFrame:RegisterEvent("PLAYER_LOGOUT")
	eFrame:RegisterEvent("ADDON_ACTION_FORBIDDEN")
	eFrame:SetScript("OnEvent", function(self, event, arg1)

		if event == "ADDON_LOADED" and arg1 == "Leatrix_Maps" then
			-- Load settings or set defaults
			LeaMapsLC:LoadVarChk("UnlockMap", "Off")					-- Unlock map frame
			LeaMapsLC:LoadVarChk("EnableMovement", "On")				-- Enable frame movement
			LeaMapsLC:LoadVarNum("MapScale", 1.0, 0.5, 2)				-- Map scale
			LeaMapsLC:LoadVarNum("MaxMapScale", 0.9, 0.5, 2)			-- Maximised map scale
			LeaMapsLC:LoadVarChk("StickyMapFrame", "Off")				-- Sticky map frame
			LeaMapsLC:LoadVarChk("RememberZoom", "On")					-- Remember zoom level
			LeaMapsLC:LoadVarChk("NoMapFade", "On")						-- Disable map fade
			LeaMapsLC:LoadVarChk("NoMapEmote", "On")					-- Disable map emote
			LeaMapsLC:LoadVarAnc("MapPosA", "TOPLEFT")					-- Windowed map anchor
			LeaMapsLC:LoadVarAnc("MapPosR", "TOPLEFT")					-- Windowed map relative
			LeaMapsLC:LoadVarNum("MapPosX", 16, -5000, 5000)			-- Windowed map X
			LeaMapsLC:LoadVarNum("MapPosY", -94, -5000, 5000)			-- Windowed map Y
			LeaMapsLC:LoadVarAnc("MaxMapPosA", "CENTER")				-- Maximised map anchor
			LeaMapsLC:LoadVarAnc("MaxMapPosR", "CENTER")				-- Maximised map relative
			LeaMapsLC:LoadVarNum("MaxMapPosX", 0, -5000, 5000)			-- Maximised map X
			LeaMapsLC:LoadVarNum("MaxMapPosY", 0, -5000, 5000)			-- Maximised map Y

			-- Elements
			LeaMapsLC:LoadVarChk("RevealMap", "On")						-- Reveal map checkbox
			LeaMapsLC:LoadVarChk("RevTint", "On")						-- Tint revealed unexplored areas
			LeaMapsLC:LoadVarNum("tintRed", 0.6, 0, 1)					-- Tint red
			LeaMapsLC:LoadVarNum("tintGreen", 0.6, 0, 1)				-- Tint green
			LeaMapsLC:LoadVarNum("tintBlue", 1, 0, 1)					-- Tint blue
			LeaMapsLC:LoadVarNum("tintAlpha", 1, 0, 1)					-- Tint transparency
			LeaMapsLC:LoadVarChk("ShowIcons", "On")						-- Location icons
			LeaMapsLC:LoadVarChk("ShowCoords", "On")					-- Show coordinates
			LeaMapsLC:LoadVarChk("CoordsLargeFont", "Off")				-- Coordinates large font
			LeaMapsLC:LoadVarChk("CoordsBackground", "On")				-- Coordinates background

			-- Settings
			LeaMapsLC:LoadVarChk("ShowMinimapIcon", "On")				-- Show minimap button

			-- Panel
			LeaMapsLC:LoadVarAnc("MainPanelA", "CENTER")				-- Panel anchor
			LeaMapsLC:LoadVarAnc("MainPanelR", "CENTER")				-- Panel relative
			LeaMapsLC:LoadVarNum("MainPanelX", 0, -5000, 5000)			-- Panel X axis
			LeaMapsLC:LoadVarNum("MainPanelY", 0, -5000, 5000)			-- Panel Y axis

			LeaMapsLC:SetDim()

			-- Set initial minimum button position
			if not LeaMapsDB["minimapPos"] then
				LeaMapsDB["minimapPos"] = 204
			end

		elseif event == "PLAYER_LOGIN" then
			-- Run main function
			LeaMapsLC:MainFunc()

		elseif event == "PLAYER_LOGOUT" and not LeaMapsLC["NoSaveSettings"] then
			-- Mechanics
			LeaMapsDB["UnlockMap"] = LeaMapsLC["UnlockMap"]
			LeaMapsDB["EnableMovement"] = LeaMapsLC["EnableMovement"]
			LeaMapsDB["MapScale"] = LeaMapsLC["MapScale"]
			LeaMapsDB["MaxMapScale"] = LeaMapsLC["MaxMapScale"]
			LeaMapsDB["StickyMapFrame"] = LeaMapsLC["StickyMapFrame"]
			LeaMapsDB["RememberZoom"] = LeaMapsLC["RememberZoom"]
			LeaMapsDB["NoMapFade"] = LeaMapsLC["NoMapFade"]
			LeaMapsDB["NoMapEmote"] = LeaMapsLC["NoMapEmote"]
			LeaMapsDB["MapPosA"] = LeaMapsLC["MapPosA"]
			LeaMapsDB["MapPosR"] = LeaMapsLC["MapPosR"]
			LeaMapsDB["MapPosX"] = LeaMapsLC["MapPosX"]
			LeaMapsDB["MapPosY"] = LeaMapsLC["MapPosY"]
			LeaMapsDB["MaxMapPosA"] = LeaMapsLC["MaxMapPosA"]
			LeaMapsDB["MaxMapPosR"] = LeaMapsLC["MaxMapPosR"]
			LeaMapsDB["MaxMapPosX"] = LeaMapsLC["MaxMapPosX"]
			LeaMapsDB["MaxMapPosY"] = LeaMapsLC["MaxMapPosY"]

			-- Elements
			LeaMapsDB["RevealMap"] = LeaMapsLC["RevealMap"]
			LeaMapsDB["RevTint"] = LeaMapsLC["RevTint"]
			LeaMapsDB["tintRed"] = LeaMapsLC["tintRed"]
			LeaMapsDB["tintGreen"] = LeaMapsLC["tintGreen"]
			LeaMapsDB["tintBlue"] = LeaMapsLC["tintBlue"]
			LeaMapsDB["tintAlpha"] = LeaMapsLC["tintAlpha"]
			LeaMapsDB["ShowIcons"] = LeaMapsLC["ShowIcons"]
			LeaMapsDB["ShowCoords"] = LeaMapsLC["ShowCoords"]
			LeaMapsDB["CoordsLargeFont"] = LeaMapsLC["CoordsLargeFont"]
			LeaMapsDB["CoordsBackground"] = LeaMapsLC["CoordsBackground"]

			-- Settings
			LeaMapsDB["ShowMinimapIcon"] = LeaMapsLC["ShowMinimapIcon"]

			-- Panel
			LeaMapsDB["MainPanelA"] = LeaMapsLC["MainPanelA"]
			LeaMapsDB["MainPanelR"] = LeaMapsLC["MainPanelR"]
			LeaMapsDB["MainPanelX"] = LeaMapsLC["MainPanelX"]
			LeaMapsDB["MainPanelY"] = LeaMapsLC["MainPanelY"]

		elseif event == "ADDON_ACTION_FORBIDDEN" and arg1 == "Leatrix_Maps" then
			-- Stop error has occured
			StaticPopup_Hide("ADDON_ACTION_FORBIDDEN")
			stopFrame:Show()

		end
	end)

	----------------------------------------------------------------------
	-- L40: Panel
	----------------------------------------------------------------------

	-- Create the panel
	local PageF = CreateFrame("Frame", nil, UIParent)

	-- Make it a system frame
	--_G["LeaMapsGlobalPanel"] = PageF
	--table.insert(UISpecialFrames, "LeaMapsGlobalPanel")

	-- Set frame parameters
	LeaMapsLC["PageF"] = PageF
	PageF:SetSize(470, 340)
	PageF:Hide()
	PageF:SetFrameStrata("FULLSCREEN_DIALOG")
	PageF:SetFrameLevel(20)
	PageF:SetClampedToScreen(true)
	PageF:EnableMouse(true)
	PageF:SetMovable(true)
	PageF:RegisterForDrag("LeftButton")
	PageF:SetScript("OnDragStart", PageF.StartMoving)
	PageF:SetScript("OnDragStop", function()
		PageF:StopMovingOrSizing()
		PageF:SetUserPlaced(false)
		-- Save panel position
		LeaMapsLC["MainPanelA"], void, LeaMapsLC["MainPanelR"], LeaMapsLC["MainPanelX"], LeaMapsLC["MainPanelY"] = PageF:GetPoint()
	end)

	-- Add background color
	PageF.t = PageF:CreateTexture(nil, "BACKGROUND")
	PageF.t:SetAllPoints()
	PageF.t:SetColorTexture(0.05, 0.05, 0.05, 0.9)

	-- Add textures
	local MainTexture = PageF:CreateTexture(nil, "BORDER")
	MainTexture:SetTexture("Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
	MainTexture:SetSize(470, 293)
	MainTexture:SetPoint("TOPRIGHT")
	MainTexture:SetVertexColor(0.7, 0.7, 0.7, 0.7)
	MainTexture:SetTexCoord(0.09, 1, 0, 1)

	local FootTexture = PageF:CreateTexture(nil, "BORDER")
	FootTexture:SetTexture("Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
	FootTexture:SetSize(470, 48)
	FootTexture:SetPoint("BOTTOM")
	FootTexture:SetVertexColor(0.5, 0.5, 0.5, 1.0)

	-- Set panel position when shown
	PageF:SetScript("OnShow", function()
		PageF:ClearAllPoints()
		PageF:SetPoint(LeaMapsLC["MainPanelA"], UIParent, LeaMapsLC["MainPanelR"], LeaMapsLC["MainPanelX"], LeaMapsLC["MainPanelY"])
	end)

	-- Add main title
	PageF.mt = PageF:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	PageF.mt:SetPoint('TOPLEFT', 16, -16)
	PageF.mt:SetText("Leatrix Maps")

	-- Add version text
	PageF.v = PageF:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	PageF.v:SetHeight(32)
	PageF.v:SetPoint('TOPLEFT', PageF.mt, 'BOTTOMLEFT', 0, -8)
	PageF.v:SetPoint('RIGHT', PageF, -32, 0)
	PageF.v:SetJustifyH('LEFT'); PageF.v:SetJustifyV('TOP')
	PageF.v:SetNonSpaceWrap(true); PageF.v:SetText(L["Version"] .. " " .. LeaMapsLC["AddonVer"])

	-- Add reload UI Button
	local reloadb = LeaMapsLC:CreateButton("ReloadUIButton", PageF, "Reload", "BOTTOMRIGHT", -16, 10, 25, "Your UI needs to be reloaded for some of the changes to take effect.|n|nYou don't have to click the reload button immediately but you do need to click it when you are done making changes and you want the changes to take effect.")
	LeaMapsLC:LockItem(reloadb, true)
	reloadb:SetScript("OnClick", ReloadUI)

	reloadb.f = reloadb:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	reloadb.f:SetHeight(32)
	reloadb.f:SetPoint('RIGHT', reloadb, 'LEFT', -10, 0)
	reloadb.f:SetText(L["Your UI needs to be reloaded."])
	reloadb.f:Hide()

	-- Add close Button
	local CloseB = CreateFrame("Button", nil, PageF, "UIPanelCloseButton") 
	CloseB:SetSize(30, 30)
	CloseB:SetPoint("TOPRIGHT", 0, 0)

	-- Add content
	LeaMapsLC:MakeTx(PageF, "Mechanics", 16, -72)
	LeaMapsLC:MakeCB(PageF, "UnlockMap", "Unlock map frame", 16, -92, true, "If checked, you will be able to move and scale the map.|n|nThe map position and scale will be saved separately for the maximised and windowed maps.")
	LeaMapsLC:MakeCB(PageF, "RememberZoom", "Remember zoom level", 16, -112, false, "If checked, opening the map will use the same zoom level from when you last closed it as long as the map zone has not changed.")
	LeaMapsLC:MakeCB(PageF, "NoMapFade", "Disable map fade", 16, -132, false, "If checked, the map will not fade while your character is moving.")
	LeaMapsLC:MakeCB(PageF, "NoMapEmote", "Disable reading emote", 16, -152, false, "If checked, your character will not perform the reading emote when you open the map.")

	LeaMapsLC:MakeTx(PageF, "Elements", 225, -72)
	LeaMapsLC:MakeCB(PageF, "RevealMap", "Show unexplored areas", 225, -92, false, "If checked, unexplored areas of the map will be shown.")
	LeaMapsLC:MakeCB(PageF, "ShowIcons", "Show dungeons and raids", 225, -112, false, "If checked, dungeons and raids will be shown.")
	LeaMapsLC:MakeCB(PageF, "ShowCoords", "Show coordinates", 225, -132, false, "If checked, coordinates will be shown.")

	LeaMapsLC:MakeTx(PageF, "Settings", 225, -172)
	LeaMapsLC:MakeCB(PageF, "ShowMinimapIcon", "Show minimap button", 225, -192, false, "If checked, the minimap button will be shown.")

 	LeaMapsLC:CfgBtn("RevTintBtn", LeaMapsCB["RevealMap"])
 	LeaMapsLC:CfgBtn("UnlockMapBtn", LeaMapsCB["UnlockMap"])
 	LeaMapsLC:CfgBtn("ShowCoordsBtn", LeaMapsCB["ShowCoords"])
